/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IACLLCT_H
#define _IACLLCT_H

#include <iglobals.h>
#include <icursor.h>

template < class Element >
class IACollection {
public:

  virtual                ~IACollection             ();

  virtual IBoolean       add                         (Element const&) = 0;

  virtual IBoolean       add                         (Element const&,
                                               ICursor&) = 0;

  virtual void           addAllFrom                  (IACollection <Element> const&);

  virtual void           copy                        (IACollection
                                                  <Element> const&);

  virtual Element const& elementAt                   (ICursor const&) const = 0;

  virtual Element&       elementAt                   (ICursor const&) = 0;

  virtual Element const& anyElement                  () const = 0;

  virtual void           removeAt                    (ICursor const&) = 0;

  virtual INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0) = 0;

  virtual void           replaceAt                   (ICursor const&,
                                               Element const&) = 0;

  virtual void           removeAll                   () = 0;

  virtual IBoolean       isBounded                   () const = 0;

  virtual INumber        maxNumberOfElements         () const = 0;

  virtual INumber        numberOfElements            () const = 0;

  virtual IBoolean       isEmpty                     () const = 0;

  virtual IBoolean       isFull                      () const = 0;

  virtual ICursor*       newCursor                   () const = 0;

  virtual IBoolean       setToFirst                  (ICursor&) const = 0;

  virtual IBoolean       setToNext                   (ICursor&) const = 0;

  virtual IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0) = 0;

  virtual IBoolean       allElementsDo               (IIterator <Element>&) = 0;

  virtual IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const = 0;

  virtual IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const = 0;

  virtual IBoolean       isConsistent                () const = 0;

protected:
  virtual void*   identity () const;
          Boolean isIdentical (IACollection < Element > const& collection) const
          { return identity () == collection.identity ();
          }
};

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <iacllct.c>
#endif
#endif

#endif
