/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IAMAP_H
#define _IAMAP_H

#include <iglobals.h>
#include <icursor.h>
#include <iaeqkey.h>

template < class Element, class Key >
class IAMap :
  public virtual IAEqualityKeyCollection < Element, Key > {
public:

  virtual IBoolean       operator ==                 (IAMap < Element, Key > const&) const;

  virtual IBoolean       operator !=                 (IAMap < Element, Key > const&) const;

  virtual void           unionWith                   (IAMap < Element, Key > const&) = 0;

  virtual void           intersectionWith            (IAMap < Element, Key > const&) = 0;

  virtual void           differenceWith              (IAMap < Element, Key > const&) = 0;

  virtual void           addUnion                    (IAMap < Element, Key > const&,
                                               IAMap < Element, Key > const&) = 0;

  virtual void           addIntersection             (IAMap < Element, Key > const&,
                                               IAMap < Element, Key > const&) = 0;

  virtual void           addDifference               (IAMap < Element, Key > const&,
                                               IAMap < Element, Key > const&) = 0;

protected:

   INumber        numberOfOccurrences         (Element const&) const;

   IBoolean       locateNext                  (Element const&, ICursor&)
                                               const;

   INumber        removeAllOccurrences        (Element const&);

   INumber        numberOfElementsWithKey     (Key const&) const;

   IBoolean       locateNextElementWithKey    (Key const&,
                                               ICursor&) const;

   INumber        removeAllElementsWithKey    (Key const&);

   INumber        numberOfDifferentKeys       () const;

   IBoolean       setToNextWithDifferentKey   (ICursor&) const;

};

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <iamap.c>
#endif
#endif

#endif
