/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IASTACK_H
#define _IASTACK_H

#include <iglobals.h>
#include <icursor.h>

template < class Element >
class IAStack {
public:

  virtual                ~IAStack               ();

  virtual IBoolean       add                    (Element const&) = 0;

  virtual IBoolean       add                    (Element const&,
                                                 ICursor&) = 0;

  virtual void           addAllFrom             (IAStack <Element> const&);

  virtual void           copy                   (IAStack <Element> const&);

  virtual Element const& elementAt              (ICursor const&) const = 0;

  virtual Element const& anyElement             () const = 0;

  virtual void           removeAll              () = 0;

  virtual IBoolean       isBounded              () const = 0;

  virtual INumber        maxNumberOfElements    () const = 0;

  virtual INumber        numberOfElements       () const = 0;

  virtual IBoolean       isEmpty                () const = 0;

  virtual IBoolean       isFull                 () const = 0;

  virtual ICursor*       newCursor              () const = 0;

  virtual IBoolean       setToFirst             (ICursor&) const = 0;

  virtual IBoolean       setToNext              (ICursor&) const = 0;

  virtual IBoolean       allElementsDo          (IBoolean (*function)
                                                  (Element const&, void*),
                                                 void* additionalArgument = 0)
                                                const = 0;

  virtual IBoolean       allElementsDo          (IConstantIterator
                                                   <Element>&) const = 0;

  virtual IBoolean       isConsistent           () const = 0;

  virtual void           removeLast             () = 0;

  virtual Element const& firstElement           () const = 0;

  virtual Element const& lastElement            () const = 0;

  virtual Element const& elementAtPosition      (IPosition) const = 0;

  virtual IBoolean       setToLast              (ICursor&) const = 0;

  virtual IBoolean       setToPrevious          (ICursor&) const = 0;

  virtual void           setToPosition          (IPosition, ICursor&) const = 0;

  virtual IBoolean       isFirst                (ICursor const&) const = 0;

  virtual IBoolean       isLast                 (ICursor const&) const = 0;

  virtual long           compare                (IAStack < Element > const&,
                                                 long (*comparisonFunction)
                                                   (Element const&,
                                                    Element const&)) const;

  virtual void           addAsLast              (Element const&) = 0;

  virtual void           addAsLast              (Element const&, ICursor&) = 0;

  virtual void           push                   (Element const&) = 0;

  virtual void           push                   (Element const&, ICursor&) = 0;

  virtual void           pop                    () = 0;

  virtual void           pop                    (Element&) = 0;

  virtual Element const& top                    () const = 0;

protected:
  virtual void* identity () const;
          Boolean isIdentical (IAStack < Element > const& collection) const
          { return identity () == collection.identity ();
          }
};

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <iastack.c>
#endif
#endif

#endif
