/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IBAGKS_H
#define _IBAGKS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element >
class IBKeyElm {
public:
  Element elem;
  INumber occurrences;
  IBKeyElm () {}
  IBKeyElm (Element const& e) : elem (e), occurrences (1) {}
  IBKeyElm (Element const& e, INumber occurrences)
  : elem (e), occurrences (occurrences) {}

  friend inline Element const& key (IBKeyElm < Element > const& e)
  { return e.elem; }
};

template < class Element, class ElementOps, class Base >
class IWBagOnKeySet {
  Base ivBase;
  INumber ivNumberOfElements;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    INumber occurrence;
    friend class IWBagOnKeySet < Element, ElementOps, Base >;
    IBoolean isFor (IWBagOnKeySet < Element, ElementOps, Base > const& c) const
    { return Base::Cursor::isFor (c.ivBase); }

  public:
    Cursor (IWBagOnKeySet < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}

    IBoolean setToFirst ();
    IBoolean setToNext  ();

    Element const& element () const
    { return Base::Cursor::element ().elem; }
    IBoolean operator== (Cursor const& cursor) const
    { return Base::Cursor::operator== (cursor) &&
             occurrence == cursor.occurrence; }
    IBoolean operator!= (Cursor const& cursor) const
    { return ! operator== (cursor); }
  };

                  IWBagOnKeySet             (INumber
                                               numberOfElements = 100);

                  IWBagOnKeySet             (IWBagOnKeySet < Element, ElementOps, Base > const&);

   IWBagOnKeySet < Element, ElementOps, Base >&
                  operator =                  (IWBagOnKeySet < Element, ElementOps, Base > const&);

                 ~IWBagOnKeySet             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWBagOnKeySet < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       containsAllFrom             (IWBagOnKeySet < Element, ElementOps, Base > const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   INumber        numberOfOccurrences         (Element const&) const;

   IBoolean       locateNext                  (Element const&, ICursor&)
                                               const;

   INumber        removeAllOccurrences        (Element const&);

   INumber        numberOfDifferentElements   () const;

   IBoolean       setToNextDifferentElement   (ICursor&) const;

   IBoolean       operator ==                 (IWBagOnKeySet < Element, ElementOps, Base > const&) const;

   IBoolean       operator !=                 (IWBagOnKeySet < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWBagOnKeySet < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWBagOnKeySet < Element, ElementOps, Base > const&);

   void           differenceWith              (IWBagOnKeySet < Element, ElementOps, Base > const&);

   void           addUnion                    (IWBagOnKeySet < Element, ElementOps, Base > const&,
                                               IWBagOnKeySet < Element, ElementOps, Base > const&);

   void           addIntersection             (IWBagOnKeySet < Element, ElementOps, Base > const&,
                                               IWBagOnKeySet < Element, ElementOps, Base > const&);

   void           addDifference               (IWBagOnKeySet < Element, ElementOps, Base > const&,
                                               IWBagOnKeySet < Element, ElementOps, Base > const&);

protected:

  void                checkCursorIsForThis (ICursor const&) const;
  void                checkCursor          (ICursor const&) const;

  struct RemoveEnv {
    IBoolean (*predicate) (Element const&, void*);
    void* env;
    INumber removeCount;
  };
  static IBoolean removePredicate (IBKeyElm < Element > const&, void*);
  static IBoolean cleanupPredicate (IBKeyElm < Element > const&, void*);

  static ElementOps elementOps ()
  { ElementOps ops;
    return ops;
  }

};

template < class Element, class ElementOps >
class IBKSOps : public ElementOps {
public:
  void           assign(IBKeyElm < Element >& e1,
                        IBKeyElm < Element > const& e2) const
                 { ElementOps::assign (e1.elem, e2.elem);
                   e1.occurrences = e2.occurrences; }
  Element const& key (IBKeyElm < Element > const& e) const
                 { return e.elem; }
  ElementOps keyOps;
};

#define IDefineGBagOnGKeySet(GKeySet, GBag) \
template < class Element, class ElementOps > \
class GKeySet ## ForGBag : \
  public GKeySet < IBKeyElm < Element >, Element, \
                   IBKSOps < Element, ElementOps > > { \
public: \
  GKeySet ## ForGBag (INumber n = 100) : \
    GKeySet < IBKeyElm < Element >, Element, \
              IBKSOps < Element, ElementOps > > (n) {} \
}; \
template < class Element, class ElementOps > \
class GBag : \
  public IWBagOnKeySet < Element, ElementOps, \
                         GKeySet ## ForGBag < Element, ElementOps > > { \
public: \
  GBag (INumber n = 100) : \
    IWBagOnKeySet < Element, ElementOps, \
                    GKeySet ## ForGBag < Element, ElementOps > > (n) {} \
};

#include <ibagks.if>

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <ibagks.c>
#endif
#endif

#endif
