/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IBAGKSS_H
#define _IBAGKSS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element >
class IBKSElm {
public:
  Element elem;
  INumber occurrences;
  IBKSElm () {}
  IBKSElm (Element const& e) : elem (e), occurrences (1) {}
  IBKSElm (Element const& e, INumber occurrences)
  : elem (e), occurrences (occurrences) {}

  friend inline Element const& key (IBKSElm < Element > const& e)
  { return e.elem; }
};

template < class Element, class ElementOps, class Base >
class IWBagOnKSSet {
  Base ivBase;
  INumber ivNumberOfElements;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    INumber occurrence;
    friend class IWBagOnKSSet < Element, ElementOps, Base >;
    IBoolean isFor (IWBagOnKSSet < Element, ElementOps, Base > const& c) const
    { return Base::Cursor::isFor (c.ivBase); }
  public:
    Cursor (IWBagOnKSSet < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}

    IBoolean setToFirst ();
    IBoolean setToNext  ();

    Element const& element () const
    { return Base::Cursor::element ().elem; }
    IBoolean operator== (Cursor const& cursor) const
    { return Base::Cursor::operator== (cursor) &&
             occurrence == cursor.occurrence; }
    IBoolean operator!= (Cursor const& cursor) const
    { return ! operator== (cursor); }
  };

                  IWBagOnKSSet             (INumber
                                               numberOfElements = 100);

                  IWBagOnKSSet             (IWBagOnKSSet < Element, ElementOps, Base > const&);

   IWBagOnKSSet < Element, ElementOps, Base >&
                  operator =                  (IWBagOnKSSet < Element, ElementOps, Base > const&);

                 ~IWBagOnKSSet             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWBagOnKSSet < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       containsAllFrom             (IWBagOnKSSet < Element, ElementOps, Base > const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   INumber        numberOfOccurrences         (Element const&) const;

   IBoolean       locateNext                  (Element const&, ICursor&)
                                               const;

   INumber        removeAllOccurrences        (Element const&);

   INumber        numberOfDifferentElements   () const;

   IBoolean       setToNextDifferentElement   (ICursor&) const;

   IBoolean       operator ==                 (IWBagOnKSSet < Element, ElementOps, Base > const&) const;

   IBoolean       operator !=                 (IWBagOnKSSet < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           differenceWith              (IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           addUnion                    (IWBagOnKSSet < Element, ElementOps, Base > const&,
                                               IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           addIntersection             (IWBagOnKSSet < Element, ElementOps, Base > const&,
                                               IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           addDifference               (IWBagOnKSSet < Element, ElementOps, Base > const&,
                                               IWBagOnKSSet < Element, ElementOps, Base > const&);

protected:

  void                checkCursorIsForThis (ICursor const&) const;
  void                checkCursor          (ICursor const&) const;

  struct RemoveEnv {
    IBoolean (*predicate) (Element const&, void*);
    void* env;
    INumber removeCount;
  };
  static IBoolean removePredicate (IBKSElm < Element > const&, void*);
  static IBoolean cleanupPredicate (IBKSElm < Element > const&, void*);

  static ElementOps elementOps ()
  { ElementOps ops;
    return ops;
  }

};

template < class Element, class ElementOps >
class IBKSSOps : public ElementOps {
public:
  void           assign(IBKSElm < Element >& e1,
                        IBKSElm < Element > const& e2) const
                 { ElementOps::assign (e1.elem, e2.elem);
                   e1.occurrences = e2.occurrences; }
  Element const& key (IBKSElm < Element > const& e) const
                 { return e.elem; }
  ElementOps keyOps;
};

#define IDefineGBagOnGKeySortedSet(GKSSet, GBag) \
template < class Element, class ElementOps > \
class GKSSet ## ForGBag : \
  public GKSSet < IBKSElm < Element >, Element, \
                  IBKSSOps < Element, ElementOps > > { \
public: \
  GKSSet ## ForGBag (INumber n = 100) : \
    GKSSet < IBKSElm < Element >, Element, \
             IBKSSOps < Element, ElementOps > > (n) {} \
}; \
template < class Element, class ElementOps > \
class GBag : \
  public IWBagOnKSSet < Element, ElementOps, \
                         GKSSet ## ForGBag < Element, ElementOps > > { \
public: \
  GBag (INumber n = 100) : \
    IWBagOnKSSet < Element, ElementOps, \
                    GKSSet ## ForGBag < Element, ElementOps > > (n) {} \
};

#include <ibagkss.if>

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <ibagkss.c>
#endif
#endif

#endif
