#ifndef _ICTLEVT_
  #define _ICTLEVT_
/*******************************************************************************
* FILE NAME: ictlevt.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IControlEvent - encapsulation of WM_CONTROL event                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   (C) Copyright IBM Corporation 1992                                         *
*   All Rights Reserved                                                        *
*   Licensed Materials = Property of IBM                                       *
*                                                                              *
*******************************************************************************/
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:

class IControlEvent : public IEvent {
typedef IEvent Inherited;
/*******************************************************************************
* The IControlEvent class represents a notification event from a control.      *
* Instances of this class are constructed and processed in many control event  *
* handler classes, such as IEditHandler.  These notification events are first  *
* dispatched to the handlers attached to the control itself, then to the       *
* handlers attached to the owner window of the control.                        *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| The only way to construct instances of this class is from an IEvent object   |
| or another IControlEvent object.                                             |
------------------------------------------------------------------------------*/
  IControlEvent ( IEvent& evt );
  IControlEvent ( const IControlEvent& evt );
 ~IControlEvent ( );

/*--------------------------- Control Information ------------------------------
| These functions provide means of querying information about the control      |
| that the event deals with:                                                   |
|   controlWindow - Returns a pointer to the control object that the event     |
|                   applies to.                                                |
|   controlId     - Returns the window ID of the control that the event        |
|                   applies to.                                                |
------------------------------------------------------------------------------*/
IWindow*
  controlWindow ( ) const;
unsigned long
  controlId     ( ) const;
};

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*----------------------------- Inline Functions -----------------------------*/
#ifndef I_NO_INLINES
  #include <ictlevt.inl>
#endif

#endif /* _ICTLEVT_ */
