#ifndef _IDIEVT_
#define _IDIEVT_
/*******************************************************************************
* FILE NAME: idievt.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IDrawItemEvent -  The base owner draw-item event information class       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#if !defined( _ICTLEVT_ )
  #include <ictlevt.hpp>
#endif
#if !defined( _IHANDLE_ )
  #include <ihandle.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:
class IRectangle;

class IDrawItemEvent : public IControlEvent {
typedef IControlEvent Inherited;
/*******************************************************************************
* The IDrawItemEvent class provides the owner-item event information for       *
* owner-draw list box.                                                         *
*******************************************************************************/
public:
/*------------------------ Constructor -----------------------------------------
| The only way to construct instances of this class is from an instance of     |
| the IEvent class.                                                            |
------------------------------------------------------------------------------*/
  IDrawItemEvent (IEvent& evt);
 ~IDrawItemEvent ( );

/*-------------------------------- Accessors -----------------------------------
| These functions provide a means of getting and setting the accessible        |
| attributes of instances of this class:                                       |
|                                                                              |
|   itemPresSpaceHandle - Returns the item presentation space.                 |
|   itemId              - Returns the item identity.                           |
|   itemRect            - Returns the rectangle of the item's size.            |
------------------------------------------------------------------------------*/
virtual IPresSpaceHandle
  itemPresSpaceHandle() const;

virtual unsigned long
  itemId() const;

virtual IRectangle
  itemRect() const;

protected:
/*----------------------------- Implementation ---------------------------------
| This function provides a means of getting the owner-draw data for the        |
| instances of this class.                                                     |
|   ownerItemData - Returns the owner-draw item's Presentation Manager         |
|                   structure.                                                 |
------------------------------------------------------------------------------*/
void*
  ownerItemData() const;

private:
/*--------------------------------- PRIVATE ----------------------------------*/
void*
  pvOwnerItem;
};

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*--------------------------------- INLINES ----------------------------------*/
#ifndef I_NO_INLINES
  #include <idievt.inl>
#endif

#endif /* _IDIEVT_ */
