#ifndef _IDMEFIT_
#define _IDMEFIT_
/*******************************************************************************
* FILE NAME: idmefit.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the declaration(s) of the class(es):                    *
*     IDMEFItem - Direct manipulation entry field item class.                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IDMITEM_
  #include <idmitem.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

/* Forward Declarations */
class IDMSourceOperation;


class IDMEFItem : public IDMItem {
typedef IDMItem
  Inherited;
/*******************************************************************************
* Objects of this class are created by entry fields when a direct manipulation *
* operation commences in an source entry field and during a direct             *
* manipulation operation when a text object is dropped on a target entry      .*
* field.                                                                       *
*                                                                              *
* Objects of this class possess the following attributes (in addition to       *
* those inherited from its base class):                                        *
*   o association with a source or target drag operation object.               *
*   o association with IDM::rmLibrary rendering mechanism and IDM::rfProcess   *
*     rendering format.  If the text size is <= 255 bytes, and the             *
*     IDM::rmFile rendering mechanism is not used, we add the IDM::rfText      *
*     rendering format.  Otherwise, we add the IDM::rfSharedMem                *
*     rendering format.                                                        *
*                                                                              *
* This class provides virtual functions that implement support for direct      *
* manipulation (i.e. drag drop) entry field text items.                        *
*******************************************************************************/
public:

/*------------------------- Constructors/Destructor ----------------------------
| Objects of this class are constructed in the following manner:               |
|   o By providing a pointer to the drag source operation.                     |
|   o By providing a reference to a generic drag item handle.                  |
------------------------------------------------------------------------------*/
  IDMEFItem   ( IDMSourceOperation    *sourceOperation );

  IDMEFItem   ( const IDMItem::Handle &dragItem );

virtual
  ~IDMEFItem  ( );


/*------------------------------ Source Items ----------------------------------
| This function generates source items:                                        |
|   generateSourceItems - Generates entry field text items based upon data in  |
|                         the source entry field window.                       |
------------------------------------------------------------------------------*/
static Boolean
  generateSourceItems    ( IDMSourceOperation *sourceOperation );

/*-------------------------------- Accessors -----------------------------------
| This function provides access to text per an entry field direct              |
| manipulation operation:                                                      |
|   object       - Returns a pointer to the item text.                         |
------------------------------------------------------------------------------*/
virtual void
 *object          ( ) const;

/*------------------------------ Drop Processing -------------------------------
| This function writes data to the target window:                              |
|   targetDrop - Writes text into entry field from drag item.                  |
------------------------------------------------------------------------------*/
virtual Boolean
  targetDrop    ( IDMTargetDropEvent & );

protected:
private: /*------------------------ PRIVATE ----------------------------------*/

}; // IDMEFItem

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#endif // _IDMEFIT_
