/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IESSEQ_H
#define _IESSEQ_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class ElementOps, class Base >
class IWEqSeqOnSeq {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    IBoolean isFor (IWEqSeqOnSeq < Element, ElementOps, Base > const& c) const
    { return Base::Cursor::isFor (c.ivBase); }

  public:
    Cursor (IWEqSeqOnSeq < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWEqSeqOnSeq             (INumber
                                               numberOfElements = 100);

                  IWEqSeqOnSeq             (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   IWEqSeqOnSeq < Element, ElementOps, Base >&
                  operator =                  (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

                 ~IWEqSeqOnSeq             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       containsAllFrom             (IWEqSeqOnSeq < Element, ElementOps, Base > const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   INumber        numberOfOccurrences         (Element const&) const;

   IBoolean       locateNext                  (Element const&, ICursor&)
                                               const;

   INumber        removeAllOccurrences        (Element const&);

   IBoolean       operator ==                 (IWEqSeqOnSeq < Element, ElementOps, Base > const&) const;

   IBoolean       operator !=                 (IWEqSeqOnSeq < Element, ElementOps, Base > const&) const;

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   IBoolean       setToLast                   (ICursor&) const;

   IBoolean       setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   IBoolean       isFirst                     (ICursor const&) const;

   IBoolean       isLast                      (ICursor const&) const;

   long           compare                     (IWEqSeqOnSeq < Element, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

   void           addAsFirst                  (Element const&);

   void           addAsFirst                  (Element const&,
                                               ICursor&);

   void           addAsLast                   (Element const&);

   void           addAsLast                   (Element const&,
                                               ICursor&);

   void           addAsNext                   (Element const&,
                                               ICursor&);

   void           addAsPrevious               (Element const&,
                                               ICursor&);

   void           addAtPosition               (IPosition,
                                               Element const&);

   void           addAtPosition               (IPosition,
                                               Element const&,
                                               ICursor&);

   void           sort                        (long (*comparisonFunction)
                                               (Element const&,
                                               Element const&));

   IBoolean       locateFirst                 (Element const&, ICursor&)
                                               const;

   IBoolean       locateLast                  (Element const&,
                                               ICursor&) const;

   IBoolean       locatePrevious              (Element const&,
                                               ICursor&) const;

protected:

  static IBoolean  isContained    (Element const&, void* env);
  static IBoolean  isNotContained (Element const&, void* env);
  static IBoolean  isEqual        (Element const&, void* env);

  static ElementOps elementOps ()
  { ElementOps ops;
    return ops;
  }

};

#define IDefineGEqualitySequenceOnGSequence(GSeq, GEqSeq) \
template < class Element, class ElementOps > \
class GEqSeq : \
  public IWEqSeqOnSeq < Element, ElementOps, \
                        GSeq < Element, ElementOps > > { \
public: \
  GEqSeq (INumber n = 100) : \
    IWEqSeqOnSeq < Element, ElementOps, \
                   GSeq < Element, ElementOps > > (n) {} \
};

#include <iesseq.if>

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <iesseq.c>
#endif
#endif

#endif
