#ifndef _IFRAMEVT_
#define _IFRAMEVT_
/*******************************************************************************
* FILE NAME: iframevt.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the declaration(s) of the class(es):                    *
*     IFrameEvent               - Common base class for frame events.          *
*     IFrameFormatEvent         - Frame format event.                          *
*     iFrameControlPointerEvent - Frame control pointer event.                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

class ISWP;
class IFrameWindow;
class IRectangle;

class IFrameEvent : public IEvent {
typedef IEvent
  Inherited;
/*******************************************************************************
* The IFrameEvent class is used to represent information about frame events    *
* in general.  Instances of classes derived from this class are created by     *
* the frame window handler and dispatched to the handler's virtual function    *
* specific to a frame event.                                                   *
*                                                                              *
* The main purpose of this class is to provide a common means of obtaining     *
* the frame window associated with the event.                                  *
*******************************************************************************/
public:
/*------------------------------- Constructor ----------------------------------
| Objects of this class are constructed from generic IEvent objects.  The      |
| event's window handle must be that of a frame window.                        |
------------------------------------------------------------------------------*/
  IFrameEvent ( const IEvent &baseEvent );
  ~IFrameEvent ( );

/*-------------------------------- Accessors -----------------------------------
| This function provides a means of getting and setting the accessible         |
| attributes of instances of this class:                                       |
|   frame - Returns a pointer to the frame window receiving this event.        |
------------------------------------------------------------------------------*/
IFrameWindow
 *frame ( ) const;

private: /*------------------------ PRIVATE ----------------------------------*/
IFrameWindow
 *pFrame;
}; // class IFrameEvent

class IFrameFormatEvent : public IFrameEvent {
typedef IFrameEvent
  Inherited;
/*******************************************************************************
* The IFrameFormatEvent class is used to represent information about a frame   *
* format event.  Instances of this class are created by the frame window       *
* handler and dispatched to the handler's formatControls function when a       *
* frame format event occurs.                                                   *
*******************************************************************************/
public:
/*------------------------------- Constructor ----------------------------------
| You can construct instances of this class from a generic IEvent with         |
| particular attributes:                                                       |
|    - The event ID must be WM_FORMATFRAME.                                    |
|    - The event's first argument must be a pointer to an array of SWP         |
|      structures with dimension equal to the frame control count returned     |
|      on the preceding WM_QUERYFRAMECTLCOUNT event.                           |
|    - The event's second argument must be a pointer to a RECTL structure in   |
|      which the frame control's size and position will be returned.           |
|                                                                              |
| The event result field will eventually contain the count of the number of    |
| frame controls processed.                                                    |
------------------------------------------------------------------------------*/
  IFrameFormatEvent ( const IEvent &baseEvent );
  ~IFrameFormatEvent ( );

/*-------------------------------- Accessors -----------------------------------
| These functions provides means of getting and setting the accessible         |
| attributes of instances of this class:                                       |
|   swpArray      - Returns a pointer to the ISWP (array) associated with      |
|                   this event.                                                |
|   clientRect    - Returns the client rectangle associated with this event.   |
|   setClientRect - Sets the client rectangle value to a given argument.       |
------------------------------------------------------------------------------*/
ISWP
 *swpArray ( ) const;

IRectangle
  clientRect ( ) const;

IFrameFormatEvent
 &setClientRect ( const IRectangle &aRect );

}; // class IFramFormatEvent

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#ifndef I_NO_INLINES
  #include <iframevt.inl>
#endif

#endif // _IFRAMEVT_
