/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IKSSSEQ_H
#define _IKSSSEQ_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class Key, class ElementOps, class Base >
class IWKSSetOnSeq {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    IBoolean isFor (IWKSSetOnSeq
                     < Element, Key, ElementOps, Base > const& c) const
    { return Base::Cursor::isFor (c.ivBase); }
  public:
    Cursor (IWKSSetOnSeq < Element, Key, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWKSSetOnSeq             (INumber
                                               numberOfElements = 100);

                  IWKSSetOnSeq             (IWKSSetOnSeq < Element, Key, ElementOps, Base > const&);

   IWKSSetOnSeq < Element, Key, ElementOps, Base >&
                  operator =                  (IWKSSetOnSeq < Element, Key, ElementOps, Base > const&);

                 ~IWKSSetOnSeq             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWKSSetOnSeq < Element, Key, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   Key const&     key                         (Element const&) const;

   IBoolean       containsElementWithKey      (Key const&) const;

   IBoolean       containsAllKeysFrom         (IWKSSetOnSeq < Element, Key, ElementOps, Base > const&) const;

   IBoolean       locateElementWithKey        (Key const&, ICursor&)
                                               const;

   IBoolean       replaceElementWithKey       (Element const&);

   IBoolean       replaceElementWithKey       (Element const&,
                                               ICursor&);

   IBoolean       locateOrAddElementWithKey   (Element const&);

   IBoolean       locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   IBoolean       addOrReplaceElementWithKey  (Element const&);

   IBoolean       addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   IBoolean       removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   IBoolean       setToLast                   (ICursor&) const;

   IBoolean       setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   IBoolean       isFirst                     (ICursor const&) const;

   IBoolean       isLast                      (ICursor const&) const;

   long           compare                     (IWKSSetOnSeq < Element, Key, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

protected:

  IBoolean         locateKey      (Key const&, ICursor&) const;

  static ElementOps elementOps ()
  { ElementOps ops;
    return ops;
  }

};

#define IDefineGKeySortedSetOnGSequence(GSeq, GKSSet) \
template < class Element, class Key, class ElementOps > \
class GKSSet : \
  public IWKSSetOnSeq < Element, Key, ElementOps, \
                        GSeq < Element, ElementOps > > { \
public: \
  GKSSet (INumber n = 100) : \
    IWKSSetOnSeq < Element, Key, ElementOps, \
                   GSeq < Element, ElementOps > > (n) {} \
};

#include <ikssseq.if>

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <ikssseq.c>
#endif
#endif

#endif
