/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IMAPKSS_H
#define _IMAPKSS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class Key, class ElementOps, class Base >
class IWMapOnKSSet {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    IBoolean isFor (IWMapOnKSSet
                     < Element, Key, ElementOps, Base >const& c) const
    { return Base::Cursor::isFor (c.ivBase); }
  public:
    Cursor (IWMapOnKSSet < Element, Key, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWMapOnKSSet             (INumber
                                               numberOfElements = 100);

                  IWMapOnKSSet             (IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   IWMapOnKSSet < Element, Key, ElementOps, Base >&
                  operator =                  (IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

                 ~IWMapOnKSSet             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       containsAllFrom             (IWMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   IBoolean       operator ==                 (IWMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   IBoolean       operator !=                 (IWMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   void           unionWith                   (IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           intersectionWith            (IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           differenceWith              (IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           addUnion                    (IWMapOnKSSet < Element, Key, ElementOps, Base > const&,
                                               IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           addIntersection             (IWMapOnKSSet < Element, Key, ElementOps, Base > const&,
                                               IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           addDifference               (IWMapOnKSSet < Element, Key, ElementOps, Base > const&,
                                               IWMapOnKSSet < Element, Key, ElementOps, Base > const&);

   Key const&     key                         (Element const&) const;

   IBoolean       containsElementWithKey      (Key const&) const;

   IBoolean       containsAllKeysFrom         (IWMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   IBoolean       locateElementWithKey        (Key const&, ICursor&)
                                               const;

   IBoolean       replaceElementWithKey       (Element const&);

   IBoolean       replaceElementWithKey       (Element const&,
                                               ICursor&);

   IBoolean       locateOrAddElementWithKey   (Element const&);

   IBoolean       locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   IBoolean       addOrReplaceElementWithKey  (Element const&);

   IBoolean       addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   IBoolean       removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

protected:

  static IBoolean  isContained    (Element const&, void* env);
  static IBoolean  isNotContained (Element const&, void* env);

  static ElementOps elementOps ()
  { ElementOps ops;
    return ops;
  }

};

#define IDefineGMapOnGKeySortedSet(GKSSet, GMap) \
template < class Element, class Key, class ElementOps > \
class GMap : \
  public IWMapOnKSSet \
           < Element, Key, ElementOps, \
             GKSSet < Element, Key, ElementOps > > { \
public: \
  GMap (INumber n = 100) : \
    IWMapOnKSSet \
      < Element, Key, ElementOps, \
        GKSSet < Element, Key, ElementOps > > (n) {} \
};

#include <imapkss.if>

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <imapkss.c>
#endif
#endif

#endif
