#ifndef _IPOPMENU_
  #define _IPOPMENU_
/******************************************************************************
* FILE NAME: ipopmenu.hpp                                                     *
*                                                                             *
* DESCRIPTION:                                                                *
*    Declaration of the class:                                                *
*      IPopUpMenu - Creates pop-up menu.                                      *
*                                                                             *
* COPYRIGHT:                                                                  *
*   Licensed Materials - Property of IBM                                      *
*   (c) Copyright IBM Corporation 1992, 1993                                  *
*   US Government Users Restricted Rights - Use duplication                   *
*   or disclosure restricted by GSA ADP Schedule Contract                     *
*   with IBM Corp.                                                            *
*                                                                             *
*******************************************************************************/
#ifndef _IMENU_
#include <imenu.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes
class   IResourceId;
class   IPoint;
class   IWindow;

class IPopUpMenu : public IMenu {
typedef IMenu
 Inherited;
/*******************************************************************************
*  The IPopUpMenu class is used to create a pop-up menu.  The library          *
*  supports the creation of pop-up menus on demand using the                   *
*  IMenuHandler::makePopUpMenu function.                                       *
*******************************************************************************/
public:
/*------------------ Constructor/Destructor ------------------------------------
|  You can construct an instance of this class in the following ways:          |
|   - By creating an empty menu bar using a window ID, frame window owner,     |
|     and styles.                                                              |
|   - By creating a menu bar with a menu resource ID and frame window owner.   |
------------------------------------------------------------------------------*/
  IPopUpMenu ( IWindow*           owner,
               unsigned long      menuWindowId,
               const Style&       style        = defaultStyle());

  IPopUpMenu ( const IResourceId& menuResId,
               IWindow*           owner );
virtual
  ~IPopUpMenu ( );

/*------------------------ Show Menu -------------------------------------------
| This function provides a mean to show the pop-up menu:                       |
|   show - Displays the pop-up menu at the given location. If                  |
|          IWindow::setAutoDeleteObject(true) is invoked, the instance of      |
|          this class is deleted when the pop-up menu terminates.              |
------------------------------------------------------------------------------*/
IPopUpMenu
  &show ( const IPoint& atLocation );

/*----------------------- Private -----------------------------------*/
private:
/*------------------- Hidden Functions. -----------------------------------
| The following inherited functions are hidden in this class.              |
|-------------------------------------------------------------------------*/
  IPopUpMenu(const IPopUpMenu&);
  IPopUpMenu &operator=(const IPopUpMenu&);
IPopUpMenu
  &show(Boolean showMenu=true);
};

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*--------------------------------- Inlines ----------------------------------*/
#ifndef I_NO_INLINES
  #include <ipopmenu.inl>
#endif

#endif  /* _IPOPMENU_ */
