/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IQUESEQ_H
#define _IQUESEQ_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class ElementOps, class Base >
class IWQueueOnSeq {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    IBoolean isFor (IWQueueOnSeq < Element, ElementOps, Base > const& c) const
    { return Base::Cursor::isFor (c.ivBase); }
  public:
    Cursor (IWQueueOnSeq < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWQueueOnSeq             (INumber
                                               numberOfElements = 100);

                  IWQueueOnSeq             (IWQueueOnSeq < Element, ElementOps, Base > const&);

   IWQueueOnSeq < Element, ElementOps, Base >&
                  operator =                  (IWQueueOnSeq < Element, ElementOps, Base > const&);

                 ~IWQueueOnSeq             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWQueueOnSeq < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element const& anyElement                  () const;

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   void           removeFirst                 ();

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   IBoolean       setToLast                   (ICursor&) const;

   IBoolean       setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   IBoolean       isFirst                     (ICursor const&) const;

   IBoolean       isLast                      (ICursor const&) const;

   long           compare                     (IWQueueOnSeq < Element, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

   void           addAsLast                   (Element const&);

   void           addAsLast                   (Element const&,
                                               ICursor&);

   void           enqueue                     (Element const&);

   void           enqueue                     (Element const&,
                                               ICursor&);

   void           dequeue                     ();

   void           dequeue                     (Element&);

};

#define IDefineGQueueOnGSequence(GSeq, GQueue) \
template < class Element, class ElementOps > \
class GQueue : \
  public IWQueueOnSeq < Element, ElementOps, GSeq < Element, ElementOps > > { \
public: \
  GQueue (INumber n = 100) : \
    IWQueueOnSeq < Element, ElementOps, \
                   GSeq < Element, ElementOps > > (n) {} \
};

#include <iqueseq.if>

#endif
