/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

template < class Element, class Base >
void* IRBag < Element, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Base >
IRBag < Element, Base >::
~IRBag ()
{
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
add (Element const& element)
{ return ivBase.add (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
add (Element const& element, ICursor &cursor)
{ return ivBase.add (element, cursor);
}

template < class Element, class Base >
Element const& IRBag < Element, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element& IRBag < Element, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element const& IRBag < Element, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Base >
void IRBag < Element, Base >::
removeAt (ICursor const& cursor)
{ ivBase.removeAt (cursor);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return ivBase.removeAll (predicate, env);
}

template < class Element, class Base >
void IRBag < Element, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{ ivBase.replaceAt (cursor, element);
}

template < class Element, class Base >
void IRBag < Element, Base >::
removeAll ()
{ ivBase.removeAll ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Base >
ICursor* IRBag < Element, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
contains (Element const& element) const
{ return ivBase.contains (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locate (Element const& element, ICursor &cursor) const
{ return ivBase.locate (element, cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locateOrAdd (Element const& element)
{ return ivBase.locateOrAdd (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locateOrAdd (Element const& element, ICursor &cursor)
{ return ivBase.locateOrAdd (element, cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
remove (Element const& element)
{ return ivBase.remove (element);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
numberOfOccurrences (Element const& element) const
{ return ivBase.numberOfOccurrences (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locateNext (Element const& element, ICursor& cursor) const
{ return ivBase.locateNext (element, cursor);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
removeAllOccurrences (Element const& element)
{ return ivBase.removeAllOccurrences (element);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
numberOfDifferentElements () const
{ return ivBase.numberOfDifferentElements ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
setToNextDifferentElement (ICursor& cursor) const
{ return ivBase.setToNextDifferentElement (cursor);
}

template < class Element, class Base >
void IRBag < Element, Base >::
unionWith (IABag < Element > const& collection)
{ if (isIdentical (collection))
    ivBase.unionWith (ivBase);
  else
    addAllFrom (collection);
}

template < class Element, class Base >
void IRBag < Element, Base >::
intersectionWith (IABag < Element > const& collection)
{ if (! isIdentical (collection))
  { IAEqualityCollection < Element > const* env = &collection;
    removeAll (IAEqualityCollection < Element >::isNotContained, &env);

    ICursor *cursor = collection.newCursor ();
    for (cursor->setToFirst ();
         cursor->isValid ();
         collection.setToNextDifferentElement (*cursor)) {
      Element const& e = collection.elementAt (*cursor);
      INumber n = numberOfOccurrences (e);
      if (n > 0) {
        INumber n1 = collection.numberOfOccurrences (e);
        for (; n > n1; n--)
          remove (e);
      }
    }
    delete cursor;

  }
}

template < class Element, class Base >
void IRBag < Element, Base >::
differenceWith (IABag < Element > const& collection)
{ if (! isIdentical (collection))
  { ICursor *cursor = collection.newCursor ();
    forCursor (*cursor)
      remove (collection.elementAt (*cursor));
    delete cursor;
  }
  else
    removeAll ();
}

template < class Element, class Base >
void IRBag < Element, Base >::
addUnion (IABag < Element > const& collection1,
          IABag < Element > const& collection2)
{ if (isIdentical (collection1)) {
    if (isIdentical (collection2))
      ivBase.addUnion (ivBase, ivBase);
    else {
      unionWith (collection1);
      unionWith (collection2);
    }
  }
  else {
    unionWith (collection2);
    unionWith (collection1);
  }
}

template < class Element, class Base >
void IRBag < Element, Base >::
addIntersection (IABag < Element > const& collection1,
                 IABag < Element > const& collection2)
{ if (isIdentical (collection1)) {
    if (isIdentical (collection2))
      ivBase.unionWith (ivBase);
    else
      addIntersection (collection2, collection1);
  }
  else {
    ICursor *cursor2 = collection2.newCursor ();
    for (cursor2->setToFirst ();
         cursor2->isValid ();
         collection2.setToNextDifferentElement (*cursor2)) {
      Element const& e = collection2.elementAt (*cursor2);
      INumber n1, n2;
      n1 = collection1.numberOfOccurrences (e);
      n2 = collection2.numberOfOccurrences (e);
      INumber n = (n1 < n2 ? n1 : n2);
      for ( ; n > 0 ; n--)
        add (e);
    }
    delete cursor2;
  }
}

template < class Element, class Base >
void IRBag < Element, Base >::
addDifference (IABag < Element > const& collection1,
               IABag < Element > const& collection2)
{ if (isIdentical (collection1)) {
    if (isIdentical (collection2))
      return;
    else {
      ivBase.unionWith (ivBase);
      ICursor *cursor2 = collection2.newCursor ();
      for (cursor2->setToFirst ();
           cursor2->isValid ();
           collection2.setToNextDifferentElement (*cursor2)) {
        Element const& e = collection2.elementAt (*cursor2);
        INumber n1, n2;
        n1 = numberOfOccurrences (e) / 2;
        n2 = collection2.numberOfOccurrences (e);
        INumber n = (n1 < n2 ? n1 : n2);
        for ( ; n > 0 ; n--)
          remove (e);
      }
      delete cursor2;
    }
  }
  else {
    ICursor *cursor1 = collection1.newCursor ();
    for (cursor1->setToFirst ();
         cursor1->isValid ();
         collection1.setToNextDifferentElement (*cursor1)) {
      Element const& e = collection1.elementAt (*cursor1);
      INumber n1, n2;
      n1 = collection1.numberOfOccurrences (e);
      n2 = collection2.numberOfOccurrences (e);
      INumber n = (n1 < n2 ? 0 : n1 - n2);
      for ( ; n > 0 ; n--)
        add (e);
    }
    delete cursor1;
  }
}

