/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IREQSEQ_H
#define _IREQSEQ_H

#include <iaeqseq.h>

template < class Element, class Base >
class IREqualitySequence : public virtual IAEqualitySequence < Element > {
  Base &ivBase;
public:

       IREqualitySequence (Base& base) : ivBase (base) {};

                 ~IREqualitySequence             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   INumber        numberOfOccurrences         (Element const&) const;

   IBoolean       locateNext                  (Element const&, ICursor&)
                                               const;

   INumber        removeAllOccurrences        (Element const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   IBoolean       setToLast                   (ICursor&) const;

   IBoolean       setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   IBoolean       isFirst                     (ICursor const&) const;

   IBoolean       isLast                      (ICursor const&) const;

   void           addAsFirst                  (Element const&);

   void           addAsFirst                  (Element const&,
                                               ICursor&);

   void           addAsLast                   (Element const&);

   void           addAsLast                   (Element const&,
                                               ICursor&);

   void           addAsNext                   (Element const&,
                                               ICursor&);

   void           addAsPrevious               (Element const&,
                                               ICursor&);

   void           addAtPosition               (IPosition,
                                               Element const&);

   void           addAtPosition               (IPosition,
                                               Element const&,
                                               ICursor&);

   void           sort                        (long (*comparisonFunction)
                                               (Element const&,
                                               Element const&));

   IBoolean       locateFirst                 (Element const&, ICursor&)
                                               const;

   IBoolean       locateLast                  (Element const&,
                                               ICursor&) const;

   IBoolean       locatePrevious              (Element const&,
                                               ICursor&) const;

protected:
  void* identity () const;
};

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <ireqseq.c>
#endif
#endif

#endif
