#ifndef _IRESLIB_INL_
#define _IRESLIB_INL_ 0
/*******************************************************************************
* FILE NAME: ireslib.inl                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of the inline functions for the          *
*   class(es) declared in ireslib.hpp.                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IRESLIB_
  #undef  _IRESLIB_INL_
  #define _IRESLIB_INL_ 1
  #include <ireslib.hpp>
#endif

#ifndef _IAPP_
  #include <iapp.hpp>
#endif

#if _IRESLIB_INL_
  #define inline
#endif

/*----------------------------- IResourceLibrary -----------------------------*/
inline IPointerHandle IResourceLibrary::loadPointer(unsigned long iconId, IBase::Boolean cached) const
/***************************************************************/
/* Load a Pointer.                                             */
/***************************************************************/
{ return loadIcon(iconId, cached); }

inline IBase::Boolean IResourceLibrary::isOpen() const
/***************************************************************/
/* Return the fact the the library is always open.             */
/***************************************************************/
{
  return true;
}

/*------------------------------- IResourceId --------------------------------*/
inline IResourceId :: IResourceId(unsigned long resourceId, const IResourceLibrary& resourceLibrary)
                     : reslibCl(resourceLibrary), ulClId(resourceId)
/***************************************************************/
/* Construct from an IResourceLibrary and Id.                  */
/***************************************************************/
{ }

inline IResourceId :: IResourceId(unsigned long resourceId, const IDynamicLinkLibrary& dllLibrary)
                     : reslibCl(dllLibrary), ulClId(resourceId)
/***************************************************************/
/* Construct from an IDynamicLinkLibrary and Id.               */
/***************************************************************/
{ }

inline IResourceId :: IResourceId(unsigned long resourceId)
                       : reslibCl ( IApplication::current().userResourceLibrary() ),
                         ulClId   ( resourceId )
/***************************************************************/
/* Construct a Resource Id from the userResourceLibrary.       */
/***************************************************************/
{ }

inline const IResourceLibrary& IResourceId::resourceLibrary() const
/***************************************************************/
/* Return the Resource Manager associated with the id.         */
/***************************************************************/
{ return reslibCl; }

inline IResourceId::operator unsigned long( )   const
/***************************************************************/
/* Return the Resource Id as an unsigned long integer.         */
/***************************************************************/
{ return this->ulClId; }

inline unsigned long IResourceId::id( )   const
/***************************************************************/
/* Return the Resource Id as an unsigned long integer.         */
/***************************************************************/
{ return this->ulClId; }

#endif // _IRESLIB_INL_
