/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IRQUEUE_H
#define _IRQUEUE_H

#include <iaqueue.h>

template < class Element, class Base >
class IRQueue : public virtual IAQueue < Element > {
  Base &ivBase;
public:

       IRQueue (Base& base) : ivBase (base) {};

                 ~IRQueue             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   Element const& elementAt                   (ICursor const&) const;

   Element const& anyElement                  () const;

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   void           removeFirst                 ();

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   IBoolean       setToLast                   (ICursor&) const;

   IBoolean       setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   IBoolean       isFirst                     (ICursor const&) const;

   IBoolean       isLast                      (ICursor const&) const;

   void           addAsLast                   (Element const&);

   void           addAsLast                   (Element const&,
                                               ICursor&);

   void           enqueue                     (Element const&);

   void           enqueue                     (Element const&,
                                               ICursor&);

   void           dequeue                     ();

   void           dequeue                     (Element&);

protected:
  void* identity () const;
};

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <irqueue.c>
#endif
#endif

#endif
