/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IRSRTMAP_H
#define _IRSRTMAP_H

#include <iasrtmap.h>

template < class Element, class Key, class Base >
class IRSortedMap : public virtual IASortedMap < Element, Key > {
  Base &ivBase;
public:

       IRSortedMap (Base& base) : ivBase (base) {};

                 ~IRSortedMap             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   void           unionWith                   (IASortedMap < Element, Key > const&);

   void           intersectionWith            (IASortedMap < Element, Key > const&);

   void           differenceWith              (IASortedMap < Element, Key > const&);

   void           addUnion                    (IASortedMap < Element, Key > const&,
                                               IASortedMap < Element, Key > const&);

   void           addIntersection             (IASortedMap < Element, Key > const&,
                                               IASortedMap < Element, Key > const&);

   void           addDifference               (IASortedMap < Element, Key > const&,
                                               IASortedMap < Element, Key > const&);

   Key const&     key                         (Element const&) const;

   IBoolean       containsElementWithKey      (Key const&) const;

   IBoolean       locateElementWithKey        (Key const&, ICursor&)
                                               const;

   IBoolean       replaceElementWithKey       (Element const&);

   IBoolean       replaceElementWithKey       (Element const&,
                                               ICursor&);

   IBoolean       locateOrAddElementWithKey   (Element const&);

   IBoolean       locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   IBoolean       addOrReplaceElementWithKey  (Element const&);

   IBoolean       addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   IBoolean       removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   IBoolean       setToLast                   (ICursor&) const;

   IBoolean       setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   IBoolean       isFirst                     (ICursor const&) const;

   IBoolean       isLast                      (ICursor const&) const;

protected:
  void* identity () const;
};

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <irsrtmap.c>
#endif
#endif

#endif
