/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

template < class Element, class Base >
void* IRSortedSet < Element, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Base >
IRSortedSet < Element, Base >::
~IRSortedSet ()
{
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
add (Element const& element)
{ return ivBase.add (element);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
add (Element const& element, ICursor &cursor)
{ return ivBase.add (element, cursor);
}

template < class Element, class Base >
Element const& IRSortedSet < Element, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element& IRSortedSet < Element, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element const& IRSortedSet < Element, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
removeAt (ICursor const& cursor)
{ ivBase.removeAt (cursor);
}

template < class Element, class Base >
INumber IRSortedSet < Element, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return ivBase.removeAll (predicate, env);
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{ ivBase.replaceAt (cursor, element);
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
removeAll ()
{ ivBase.removeAll ();
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Base >
INumber IRSortedSet < Element, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Base >
INumber IRSortedSet < Element, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Base >
ICursor* IRSortedSet < Element, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
contains (Element const& element) const
{ return ivBase.contains (element);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
locate (Element const& element, ICursor &cursor) const
{ return ivBase.locate (element, cursor);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
locateOrAdd (Element const& element)
{ return ivBase.locateOrAdd (element);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
locateOrAdd (Element const& element, ICursor &cursor)
{ return ivBase.locateOrAdd (element, cursor);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
remove (Element const& element)
{ return ivBase.remove (element);
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
unionWith (IASortedSet < Element > const& collection)
{ if (isIdentical (collection))
    ivBase.unionWith (ivBase);
  else
    addAllFrom (collection);
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
intersectionWith (IASortedSet < Element > const& collection)
{ if (! isIdentical (collection))
  { IAEqualityCollection < Element > const* env = &collection;
    removeAll (IAEqualityCollection < Element >::isNotContained, &env);

  }
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
differenceWith (IASortedSet < Element > const& collection)
{ if (! isIdentical (collection))
  { ICursor *cursor = collection.newCursor ();
    forCursor (*cursor)
      remove (collection.elementAt (*cursor));
    delete cursor;
  }
  else
    removeAll ();
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
addUnion (IASortedSet < Element > const& collection1,
          IASortedSet < Element > const& collection2)
{ unionWith (collection1);
  unionWith (collection2);
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
addIntersection (IASortedSet < Element > const& collection1,
                 IASortedSet < Element > const& collection2)
{ if (isIdentical (collection1) || isIdentical (collection2))
    return;
  else {
    ICursor *cursor1 = collection1.newCursor ();
    forCursor (*cursor1) {
      if (collection2.contains (collection1.elementAt (*cursor1)))
        add (collection1.elementAt (*cursor1));
    }
    delete cursor1;
  }
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
addDifference (IASortedSet < Element > const& collection1,
               IASortedSet < Element > const& collection2)
{ if (isIdentical (collection1))
    return;
  else if (isIdentical (collection2))
    unionWith (collection1);
  else {
    ICursor *cursor1 = collection1.newCursor ();
    forCursor (*cursor1) {
      if (! collection2.contains (collection1.elementAt (*cursor1)))
        add (collection1.elementAt (*cursor1));
    }
    delete cursor1;
  }
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
removeFirst ()
{ ivBase.removeFirst ();
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
removeLast ()
{ ivBase.removeLast ();
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
removeAtPosition (IPosition position)
{ ivBase.removeAtPosition (position);
}

template < class Element, class Base >
Element const& IRSortedSet < Element, Base >::
firstElement () const
{ return ivBase.firstElement ();
}

template < class Element, class Base >
Element const& IRSortedSet < Element, Base >::
lastElement () const
{ return ivBase.lastElement ();
}

template < class Element, class Base >
Element const& IRSortedSet < Element, Base >::
elementAtPosition (IPosition position) const
{ return ivBase.elementAtPosition (position);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
setToLast (ICursor &cursor) const
{ return ivBase.setToLast (cursor);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
setToPrevious (ICursor &cursor) const
{ return ivBase.setToPrevious (cursor);
}

template < class Element, class Base >
void IRSortedSet < Element, Base >::
setToPosition (IPosition position, ICursor &cursor) const
{ ivBase.setToPosition (position, cursor);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
isFirst (ICursor const& cursor) const
{ return ivBase.isFirst (cursor);
}

template < class Element, class Base >
IBoolean IRSortedSet < Element, Base >::
isLast (ICursor const& cursor) const
{ return ivBase.isLast (cursor);
}

