#ifndef _ISCRLEVT_
  #define _ISCRLEVT_
/*******************************************************************************
* FILE NAME: iscrlevt.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IScrollEvent - Represents a scroll event for an IScrollBar object.       *
*                                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:
class IScrollBar;


class IScrollEvent : public IEvent {
typedef IEvent
  Inherited;
/*******************************************************************************
* The IScrollEvent class represents a scroll event for an IScrollBar object.   *
* This event is a notification for interactions with an IScrollBar object.     *
* An IScrollEvent object is created by an IScrollHandler object and passed to  *
* one of its virtual functions for processing.                                 *
*******************************************************************************/
public:
/*------------------------------- Constructor ----------------------------------
| The only way to construct instances of this class is from an IEvent          |
| instance.  This is the constructor used by the IScrollHandler class.         |
------------------------------------------------------------------------------*/
  IScrollEvent ( IEvent& event );
virtual
 ~IScrollEvent ( );

/*-------------------------------- Accessors -----------------------------------
| The following functions return information related to the scroll event:      |
|   scrollBarWindow      - Returns a pointer to the IScrollBar object to       |
|                          which the scroll event applies.                     |
|   scrollAmount         - Returns the number of units that the scroll event   |
|                          wants the scroll box to scroll.  A negative value   |
|                          means a backwards scroll event occurred.            |
|   newScrollBoxPosition - Returns the new position of the scroll box.  This   |
|                          is valid only for a scroll event passed to the      |
|                          IScrollHandler::scrollBoxTrack or                   |
|                          IScrollHandler::scrollBoxTrackEnd functions.        |
------------------------------------------------------------------------------*/
IScrollBar*
  scrollBarWindow      ( ) const;
long
  scrollAmount         ( ) const;
unsigned long
  newScrollBoxPosition ( ) const;

private:
/*--------------------------------- Private ----------------------------------*/
unsigned long
  ulClPrevScrollBoxPos;
};

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#endif /* _ISCRLEVT_ */
