#ifndef _ISETBUT_
  #define _ISETBUT_
/*******************************************************************************
* FILE NAME: isetbut.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     ISetttingButtton - This is an abstract base class for the I3StateCheckBox,*
*                ICheckBox, and IRadioButton control window classes.           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IBUTTON_
  #include <ibutton.hpp>
#endif

// Forward declarations for other classes:
class IRectangle;
#ifndef _IBITFLAG_
  #include <ibitflag.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

class ISettingButton : public IButton  {
typedef IButton Inherited;
/*******************************************************************************
* The ISettingButton class is an abstract base class for the I3StateCheckBox,  *
* ICheckBox, and IRadioButton control window classes.                          *
*                                                                              *
* An instance of this class can not be created.                                *
*                                                                              *
*******************************************************************************/
public:
/*------------------------ Constructors ----------------------------------------
| An instance of this class cannot be created.                                 |
------------------------------------------------------------------------------*/
  ISettingButton();
  virtual ~ISettingButton();

/*------------------------ Test/Select Button ----------------------------------
| isSelected - Returns true if the button is selected.                         |
| select     - Selects the button.                                             |
| deselect   - Deselects the button.                                           |
------------------------------------------------------------------------------*/
Boolean
  isSelected() const;
ISettingButton
 &select(),
 &deselect();

/*------------------------ Test/Set Auto Select State --------------------------
| enableAutoSelect  - A virtual declaration for enabling the automatic         |
|                     selection style for a button.                            |
| disableAutoSelect - A virtual declaration for disabling the automatic        |
|                     selection style for a button.                            |
| isAutoSelect      - A virtual declaration for querying whether the           |
|                     automatic selection style is set for a button.           |
------------------------------------------------------------------------------*/
virtual ISettingButton&
  enableAutoSelect(Boolean enable) = 0;
virtual ISettingButton&
  disableAutoSelect()  = 0;
virtual Boolean
  isAutoSelect() const = 0;

private:
  ISettingButton(const ISettingButton&);
  ISettingButton& operator=(const ISettingButton&);
};  // class ISettingButton

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#endif  /* _ISETBUT_ */
