/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _ISETKSS_H
#define _ISETKSS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class ElementOps, class Base >
class IWSetOnKSSet {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    IBoolean isFor (IWSetOnKSSet < Element, ElementOps, Base > const& c) const
    { return Base::Cursor::isFor (c.ivBase); }
  public:
    Cursor (IWSetOnKSSet < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWSetOnKSSet             (INumber
                                               numberOfElements = 100);

                  IWSetOnKSSet             (IWSetOnKSSet < Element, ElementOps, Base > const&);

   IWSetOnKSSet < Element, ElementOps, Base >&
                  operator =                  (IWSetOnKSSet < Element, ElementOps, Base > const&);

                 ~IWSetOnKSSet             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWSetOnKSSet < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       containsAllFrom             (IWSetOnKSSet < Element, ElementOps, Base > const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   IBoolean       operator ==                 (IWSetOnKSSet < Element, ElementOps, Base > const&) const;

   IBoolean       operator !=                 (IWSetOnKSSet < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           differenceWith              (IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           addUnion                    (IWSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           addIntersection             (IWSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           addDifference               (IWSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSetOnKSSet < Element, ElementOps, Base > const&);

protected:

  static IBoolean  isContained    (Element const&, void* env);
  static IBoolean  isNotContained (Element const&, void* env);

};

#define IDefineGSetOnGKeySortedSet(GKSSet, GSet) \
template < class Element, class ElementOps > \
class GKSSet ## ForGSet : \
  public GKSSet < Element, Element, \
                  IOpsWithKey < Element, ElementOps > > { \
public: \
  GKSSet ## ForGSet (INumber n = 100) : \
    GKSSet < Element, Element, \
             IOpsWithKey < Element, ElementOps > > (n) {} \
}; \
template < class Element, class ElementOps > \
class GSet : \
  public IWSetOnKSSet < Element, ElementOps, \
                        GKSSet ## ForGSet < Element, ElementOps > > { \
public: \
  GSet (INumber n = 100) : \
    IWSetOnKSSet < Element, ElementOps, \
                   GKSSet ## ForGSet < Element, ElementOps > > (n) {} \
};

#include <isetkss.if>

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <isetkss.c>
#endif
#endif

#endif
