/*******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM C/C++ Tools Version 2.01 - Collection Class Library                    *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _ISMKSS_H
#define _ISMKSS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class Key, class ElementOps, class Base >
class IWSrtMapOnKSSet {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  protected:
    IBoolean isFor (IWSrtMapOnKSSet
                     < Element, Key, ElementOps, Base >const& c) const
    { return Base::Cursor::isFor (c.ivBase); }
  public:
    Cursor (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWSrtMapOnKSSet             (INumber
                                               numberOfElements = 100);

                  IWSrtMapOnKSSet             (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   IWSrtMapOnKSSet < Element, Key, ElementOps, Base >&
                  operator =                  (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

                 ~IWSrtMapOnKSSet             ();

   IBoolean       add                         (Element const&);

   IBoolean       add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (IBoolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   IBoolean       isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   IBoolean       isEmpty                     () const;

   IBoolean       isFull                      () const;

   ICursor*       newCursor                   () const;

   IBoolean       setToFirst                  (ICursor&) const;

   IBoolean       setToNext                   (ICursor&) const;

   IBoolean       allElementsDo               (IBoolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   IBoolean       allElementsDo               (IIterator <Element>&);

   IBoolean       allElementsDo               (IBoolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   IBoolean       allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   IBoolean       isConsistent                () const;

   IBoolean       contains                    (Element const&) const;

   IBoolean       containsAllFrom             (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   IBoolean       locate                      (Element const&, ICursor&)
                                               const;

   IBoolean       locateOrAdd                 (Element const&);

   IBoolean       locateOrAdd                 (Element const&,
                                               ICursor&);

   IBoolean       remove                      (Element const&);

   IBoolean       operator ==                 (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   IBoolean       operator !=                 (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   void           unionWith                   (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           intersectionWith            (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           differenceWith              (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           addUnion                    (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&,
                                               IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           addIntersection             (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&,
                                               IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   void           addDifference               (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&,
                                               IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&);

   Key const&     key                         (Element const&) const;

   IBoolean       containsElementWithKey      (Key const&) const;

   IBoolean       containsAllKeysFrom         (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&) const;

   IBoolean       locateElementWithKey        (Key const&, ICursor&)
                                               const;

   IBoolean       replaceElementWithKey       (Element const&);

   IBoolean       replaceElementWithKey       (Element const&,
                                               ICursor&);

   IBoolean       locateOrAddElementWithKey   (Element const&);

   IBoolean       locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   IBoolean       addOrReplaceElementWithKey  (Element const&);

   IBoolean       addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   IBoolean       removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   IBoolean       setToLast                   (ICursor&) const;

   IBoolean       setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   IBoolean       isFirst                     (ICursor const&) const;

   IBoolean       isLast                      (ICursor const&) const;

   long           compare                     (IWSrtMapOnKSSet < Element, Key, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

protected:

  static IBoolean  isContained    (Element const&, void* env);
  static IBoolean  isNotContained (Element const&, void* env);

  static ElementOps elementOps ()
  { ElementOps ops;
    return ops;
  }

};

#define IDefineGSortedMapOnGKeySortedSet(GKSSet, GSrtMap) \
template < class Element, class Key, class ElementOps > \
class GSrtMap : \
  public IWSrtMapOnKSSet \
           < Element, Key, ElementOps, \
             GKSSet < Element, Key, ElementOps > > { \
public: \
  GSrtMap (INumber n = 100) : \
    IWSrtMapOnKSSet \
      < Element, Key, ElementOps, \
        GKSSet < Element, Key, ElementOps > > (n) {} \
};

#include <ismkss.if>

#ifdef __IBMCPP__
#ifndef __TEMPINC__
#include <ismkss.c>
#endif
#endif

#endif
