/******************************************************************************/
/* DDE CLIENT SAMPLE PROGRAM: Class Implementation (ACLIENT.CPP)              */
/*                                                                            */
/* COPYRIGHT: Copyright (C) International Business Machines Corp., 1992,1993. */
/*                                                                            */
/* DISCLAIMER OF WARRANTIES:                                                  */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is not part of any standard IBM product   */
/*   and is provided to you solely for the purpose of assisting you in the    */
/*   development of your applications.  The code is provided "AS IS",         */
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */
/******************************************************************************/
// NOTE: WE RECOMMEND USING A FIXED SPACE FONT TO LOOK AT THE SOURCE

#include <imenubar.hpp>                 //IMenuBar Class
#include <imle.hpp>                     //IMultiLineEdit (MLE) Class
#include <istattxt.hpp>                 //IStatic Text
#include <iapp.hpp>                     //IApplication Classes
#include <istring.hpp>                  //IString Class
#include <iddeevt.hpp>                  //IDDEEvent Classes
#include <iinfoa.hpp>                   //IInfoArea Class
#include <isplitcv.hpp>                 //ISplitCanvas Class
#include <ifont.hpp>                    //IFont Class
#include <imsgbox.hpp>                  //IMessageBox Class
#include <icolor.hpp>                   //IColor

#include "aclient.h"
#include "aclient.hpp"

//*************************************************************************
// main  - Application entry point
//*************************************************************************
void  main(int argc,char **argv)
{
   AClientWindow clientWindow(WND_CLIENT);//Create main window

   IApplication::current().run();       //Display main window & start events
} /* end main */

//**************************************************************************
// AClientWindow :: AClientWindow - Constructor for our main window
//**************************************************************************
AClientWindow :: AClientWindow(unsigned long windowId)
  : IFrameWindow (windowId),            //Call IFrameWindow constructor
    menuBar(WND_CLIENT, this),
    statusLine(WND_STATUS, this, this), //Create status line
    todoStatus(WND_TODOSTA, &statusLine, //Create todo status line
      &statusLine),
    workStatus(WND_WORKSTA, &statusLine,//Create work status line
      &statusLine),
    doneStatus(WND_DONESTA, &statusLine,//Create done status line
      &statusLine),
    cArea(WND_CAREA, this, this),       //Create Client Area
    workWindow(WND_WORKW, &cArea, this),//Create work window
    mle(WND_MLE, &cArea, &cArea),       //Create MLE to display DDE activities
    infoArea(this)                      //Create Information Area
{
  handleEventsFor(this);                //Set self as command event handler
  conversation = 0;

  cArea.setOrientation(                 //Set the orientation
    ISplitCanvas::horizontalSplit);     //  to horizontal
  setClient(&cArea);                    //Set Client to cArea

  addExtension(&statusLine,             //Add Status Line above the client
    IFrameWindow::aboveClient,          //  and specify the height
    IFont(&statusLine).maxCharHeight());//  and specify height

  setExtensionSize(&infoArea,           //
    (int)IFont(&infoArea).maxCharHeight());//and specify height
  infoArea.setInactiveText(STR_HOTNO);  //Set information area text from RC

  todoCount=0;
  workCount=0;
  doneCount=0;

  menuBar.disableItem(MI_END_CONV);     //Disable End Conversation
  menuBar.disableItem(MI_REQUEST_DATA); //Disable request data
  menuBar.disableItem(MI_POKE_DATA);    //Disable poke data
  menuBar.disableItem(MI_BEGIN_HOTLINK);//Disable begin hot link
  menuBar.disableItem(MI_END_HOTLINK);  //Disable end hot link

  workWindow.setAlignment(              //Set Alignment to Center in both
    IStaticText::centerCenter);         //  directions
  IFont tempFont("Helv", 16);           //Create Temp. Font
  workWindow.setFont(tempFont);         //Set Font for Work Item Text

  sizeTo(ISize(400,300));               //Set the size of main window
  updateStatus();                       //Update status line
  setFocus();                           //Set Focus to main window
  show();

} /* end AClientWindow :: AClientWindow(...) */

//**************************************************************************
// AClientWindow :: AClientWindow - Destructor for our main window
//**************************************************************************
AClientWindow :: ~AClientWindow()
{
  if (conversation)
  {
    try
    {
      conversation->endHotLinks();
    }
    catch (IException& exc)             //Catch Error
    {
      delete conversation;
    }
    delete conversation;
  } /* endif */
} /* end AClientWindow :: ~AClientWindow() */

//**************************************************************************
// AClientWindow :: command(..)
//**************************************************************************
Boolean AClientWindow::command(ICommandEvent& cmdevt)
{
  switch(cmdevt.commandId())
  {
    case MI_START_CONV:
    {
      conversation=new AConversation(this);
      log("<<begin: sent to Server\n");
      conversation->begin(
        "DDE Sample App",
        "Work Topic");
      menuBar.disableItem(MI_START_CONV);//Disable Start Conversation
      menuBar.enableItem(MI_END_CONV);  //Enable End Conversation
      menuBar.enableItem(MI_REQUEST_DATA);//Enable request data
      menuBar.enableItem(MI_POKE_DATA); //Enable poke data
      menuBar.enableItem(MI_BEGIN_HOTLINK);//Enable begin hot link
      break;
    }
    case MI_END_CONV:
    {
      menuBar.enableItem(MI_START_CONV);//Enable Start Conversation
      menuBar.disableItem(MI_END_CONV);  //Disable End Conversation
      menuBar.disableItem(MI_REQUEST_DATA);//Enable request data
      menuBar.disableItem(MI_POKE_DATA); //Enable poke data
      if (menuBar.isItemDisabled(MI_BEGIN_HOTLINK))
      {
        log("<<endHotLinks sent to Server\n");
        conversation->endHotLinks();
      }
      log("<<end sent to Server\n");
      conversation->end();
      menuBar.disableItem(MI_BEGIN_HOTLINK);//Disable begin hot link
      menuBar.disableItem(MI_END_HOTLINK);//Disable end hot link
      infoArea.setInactiveText(STR_HOTNO);//Set information area text from RC
      todoStatus.setColor(              //Set todo status color
        IStaticText::foreground,        //
        IColor(IColor::blue));          //
      workStatus.setColor(              //Set work status color
        IStaticText::foreground,        //
        IColor(IColor::blue));          //
      doneStatus.setColor(              //Set done status color
        IStaticText::foreground,        //
        IColor(IColor::blue));          //
      break;
    }
    case MI_REQUEST_DATA:
    {
      workItemDone();                   //Set work item done
      log("<<requestData: NextTodo sent to Server\n");
      try
      {
        conversation->requestData("NextTodo");
      }
      catch (IException& exc)           //Catch Error
      {                                 //
        IMessageBox mbox(this);         //Create Message Box
        mbox.show(exc);                 //
        return false;                   //Return error
      }                                 //
      break;
    }
    case MI_POKE_DATA:
    {
      workItemDone();                   //Set work item done
      break;
    }
    case MI_BEGIN_HOTLINK:
    {
      log("<<beginHotLink: HotTodo sent to Server\n");
      conversation->beginHotLink("HotTodo");
      log("<<beginHotLink: HotWork sent to Server\n");
      conversation->beginHotLink("HotWork");
      log("<<beginHotLink: HotDone sent to Server\n");
      conversation->beginHotLink("HotDone");
      menuBar.disableItem(MI_BEGIN_HOTLINK);//Disable begin hot link
      menuBar.enableItem(MI_END_HOTLINK);  //Enable end hot link
      infoArea.setInactiveText(STR_HOTYES);//Set information area text from RC
      todoStatus.setColor(              //Set todo status color
        IStaticText::foreground,        //
        IColor(IColor::red));           //
      workStatus.setColor(              //Set work status color
        IStaticText::foreground,        //
        IColor(IColor::red));           //
      doneStatus.setColor(              //Set done status color
        IStaticText::foreground,        //
        IColor(IColor::red));           //
      conversation->requestData("HotTodo");
      conversation->requestData("HotWork");
      conversation->requestData("HotDone");
      break;
    }
    case MI_END_HOTLINK:
    {
      log("<<endHotLinks sent to Server\n");
      conversation->endHotLinks();
      menuBar.enableItem(MI_BEGIN_HOTLINK);//Enable begin hot link
      menuBar.disableItem(MI_END_HOTLINK);//Disable end hot link
      infoArea.setInactiveText(STR_HOTNO);  //Set information area text from RC
      todoStatus.setColor(              //Set todo status color
        IStaticText::foreground,        //
        IColor(IColor::blue));          //
      workStatus.setColor(              //Set work status color
        IStaticText::foreground,        //
        IColor(IColor::blue));          //
      doneStatus.setColor(              //Set done status color
        IStaticText::foreground,        //
        IColor(IColor::blue));          //
      break;
    }
    default:
    {
      return false;
    }
  }
  return true;
} /* end AClientWindow :: command(..) */

//**************************************************************************
// AClientWindow :: log(..)
//**************************************************************************
Boolean AClientWindow::log(char * message)
{
  mle.add(message, 0,                   //Add to mle message log
    IMultiLineEdit::noTran);
  return true;
} /* end AClientWindow :: log(..) */

//**************************************************************************
// AClientWindow :: workItemDone(..)
//**************************************************************************
Boolean AClientWindow::workItemDone()
{
  IString workItem(workWindow.text());
  if (workItem.size() > 1)
  {
    log("<<pokeData: DonePoke sent to Server\n");
    conversation->pokeData(
      "DonePoke", (char*)workItem,
       workItem.size());
    setWorkItem("");
  } /* endif */
  return true;
} /* end AClientWindow :: workItemDone(..) */

//**************************************************************************
// AClientWindow :: setWorkItem(..)
//**************************************************************************
Boolean AClientWindow::setWorkItem(char * message)
{
  workWindow.setText(message);          //Update new work item
  return true;
} /* end AClientWindow :: setWorkItem(..) */

//**************************************************************************
// AClientWindow :: workItem(..)
//**************************************************************************
IString AClientWindow::workItem()
{
  return IString(workWindow.text());
} /* end AClientWindow :: setWorkItem(..) */

//**************************************************************************
// AClientWindow :: updateStatus()
//**************************************************************************
Boolean AClientWindow::updateStatus()
{
  todoStatus.setText(                   //Set the status to todo count
    IString("Todo count: ")+
    IString(todoCount));
  workStatus.setText(                   //Set the status to work count
    IString("Work count: ")+
    IString(workCount));
  doneStatus.setText(                   //Set the status to done count
    IString("Done count: ")+
    IString(doneCount));
  return true;
} /* end AClientWindow :: updateStatus() */

//**************************************************************************
// AClientWindow :: updateTodo(..)
//**************************************************************************
Boolean AClientWindow::updateTodo(char * newCount)
{
  todoStatus.setText(                   //Set the status to todo count
    IString("Todo count: ")+
    IString(newCount));
  return true;
} /* end AClientWindow :: updateTodo(..) */

//**************************************************************************
// AClientWindow :: updateWork(..)
//**************************************************************************
Boolean AClientWindow::updateWork(char * newCount)
{
  workStatus.setText(                   //Set the status to work count
    IString("Work count: ")+
    IString(newCount));
  return true;
} /* end AClientWindow :: updateWork(..) */

//**************************************************************************
// AClientWindow :: updateDone(..)
//**************************************************************************
Boolean AClientWindow::updateDone(char * newCount)
{
  doneStatus.setText(                   //Set the status to done count
    IString("Done count: ")+
    IString(newCount));
  return true;
} /* end AClientWindow :: updateDone(..) */

//**************************************************************************
// AConversation :: AConversation - Constructor for our conversation
//**************************************************************************
AConversation::AConversation(AClientWindow* frameWin)
              : IDDEClientConversation(false),
                clientWindow(frameWin) {;}

//**************************************************************************
// AConversation :: data(..)
//**************************************************************************
Boolean AConversation :: data(IDDEDataEvent& event)
{
  IString newWorkItem;
  IString newData(event.data());
  IString item(event.item());

  if (event.isDataFromHotLink())        //Is Data from hot link?
  {
    mainWindow()->log(IString(">>data hotlink(")
      + item + IString("):")
      + newData
      + IString(" received from Server\n"));
  }
  else                                  //New work item from server
  {
    mainWindow()->log(IString(">>data(")
      + item + IString("):")
      + newData
      + IString(" received from Server\n"));
  } /* endif */

  if (item == "NextTodo")             //Is item the Next Todo?
  {
    mainWindow()->setWorkItem(newData);
  } /* endif */
  if (item == "HotTodo")              //Is item the todo counter?
  {
    mainWindow()->updateTodo(newData);
  } /* endif */
  if (item == "HotWork")              //Is item the work counter?
  {
    mainWindow()->updateWork(newData);
  } /* endif */
  if (item == "HotDone")              //Is item the done counter?
  {
    mainWindow()->updateDone(newData);
  } /* endif */
  return true;
} /* end AConversation :: data(..) */

void AConversation :: conversationEnded(IDDEClientEndEvent& endEvt)
/**********************************************************/
/* Do any cleanup on close of conversation by server      */
/**********************************************************/
{
   if (endEvt.sourceOfEnd() == IDDEEndEvent::server)
     mainWindow()->log(IString(">>conversationEnded: for topic: ") +
       IString(endEvt.topic()) + IString(" in application: ") +
       IString(endEvt.application()) + IString(" by Server\n"));
   else if (endEvt.sourceOfEnd() == IDDEEndEvent::client)
     mainWindow()->log(IString(">>conversationEnded: for topic: ") +
       IString(endEvt.topic()) + IString(" in application: ") +
       IString(endEvt.application()) + IString(" by Client\n"));
   else
     mainWindow()->log(IString(">>conversationEnded: for topic: ") +
       IString(endEvt.topic()) + IString(" in application: ") +
       IString(endEvt.application()) + IString(" by error\n"));
}

/**********************************************************/
/* Handle Ack Events                                      */
/**********************************************************/
void AConversation :: acknowledged(IDDEClientAcknowledgeEvent& event)
{
   mainWindow()->log(IString(
     ">>acknowledged: received from Server\n"));
}
