/******************************************************************************/
/* DDE SERVER SAMPLE PROGRAM: Class Implemenation (ASERVER.CPP)               */
/*                                                                            */
/* COPYRIGHT: Copyright (C) International Business Machines Corp., 1992,1993. */
/*                                                                            */
/* DISCLAIMER OF WARRANTIES:                                                  */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is not part of any standard IBM product   */
/*   and is provided to you solely for the purpose of assisting you in the    */
/*   development of your applications.  The code is provided "AS IS",         */
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */
/******************************************************************************/
// NOTE: WE RECOMMEND USING A FIXED SPACE FONT TO LOOK AT THE SOURCE

#include <imle.hpp>                     //IMultiLineEdit Class
#include <istattxt.hpp>                 //IStaticText Class
#include <iinfoa.hpp>                   //IInfoArea Class
#include <ilistbox.hpp>                 //IListBox Class
#include <isplitcv.hpp>                 //ISplitCanvas Class
#include <iapp.hpp>                     //IApplication Classes
#include <ifont.hpp>                    //IFont Class
#include <imsgbox.hpp>                  //IMessageBox Class
#include <istring.hpp>                  //IString Class
#include <iddeevt.hpp>                  //IDDEEvent Classes

#include "aserver.h"
#include "aserver.hpp"                  //Include our classes

//*************************************************************************
// main  - Application entry point
//*************************************************************************
void  main(int argc,char **argv)
{
  AServerWindow serverWindow(WND_SERVER);//Create main window

  IApplication::current().run();        //Start event handling
} /* end main */

//**************************************************************************
// AServerWindow :: AServerWindow - Constructor for our main window
//**************************************************************************
AServerWindow :: AServerWindow(unsigned long windowId)
  : IFrameWindow (windowId),            //Call IFrameWindow constructor
    statusLine(WND_STATUS, this, this), //Create status line
    todoStatus(WND_TODOSTA, &statusLine,//Create todo status line
      &statusLine),
    workStatus(WND_WORKSTA, &statusLine,//Create work status line
      &statusLine),
    doneStatus(WND_DONESTA, &statusLine,//Create done status line
      &statusLine),
    cArea(WND_CAREA, this, this),       //Create Client Area
    dArea(WND_DAREA, &cArea, this),     //Create Data Area Cannvas
    todoList(WND_TODO, &dArea, this,    //Create todo list
      IRectangle(),                     //
      (IListBox::defaultStyle()         //  Use default style with
      | IListBox::noAdjustPosition)),   //  Canvas controling size
    workList(WND_WORK, &dArea, this,    //Create working list
      IRectangle(),                     //
      (IListBox::defaultStyle()         //
      | IListBox::noAdjustPosition)),   //  Allow the Canvas to control size
    doneList(WND_DONE, &dArea, this,    //Create done list
      IRectangle(),                     //
      (IListBox::defaultStyle()         //  Use default style with
      | IListBox::noAdjustPosition)),   //  Canvas controling size
    mle(WND_MLE, &cArea, &cArea),       //Create MLE to display DDE activities
    infoArea(this),                     //Create Information Area
    server("DDE Sample App",            //Create DDE Server (application name,
      "Work Topic", this)               //  topic name, window)
{
  hot=0;                                //Hot link not active
  cArea.setOrientation(                 //Set the orientation
    ISplitCanvas::horizontalSplit);     //  to horizontal
  setClient(&cArea);                    //Set Client to cArea

  addExtension(&statusLine,             //Add Status Line above the client
    IFrameWindow::aboveClient,          //  and specify the height
    IFont(&statusLine).maxCharHeight());//  and specify height
  setExtensionSize(&infoArea,           //
    (int)IFont(&infoArea).maxCharHeight());//and specify height
  infoArea.setInactiveText(STR_HOTNO);  //Set information area text from RC

  sizeTo(ISize(600,400));               //Set the size of main window
  moreTodo();                           //Initialize the todo list
  updateStatus();                       //Update status line
  setFocus();                           //Set Focus to main window
  show();
} /* end AServerWindow :: AServerWindow(...) */

//**************************************************************************
// AServerWindow :: log(..)
//**************************************************************************
Boolean AServerWindow::log(char * message)
{
  mle.add(message, 0,                   //Add to mle message log
    IMultiLineEdit::noTran);
  return true;
} /* end AServerWindow :: log(..) */

//**************************************************************************
// AServerWindow :: addDone(..)
//**************************************************************************
Boolean AServerWindow::addDone(char * newDone)
{
  doneList.addAsLast(newDone);          //Add new done item
  try
  {
    unsigned long index=                //Locate this item in the work list
      workList.locateText(newDone);
    workList.remove(index);             //Remove from work list
  }
  catch (IException& exc)               //Catch Error
  {                                     //
    IMessageBox mbox(this);             //Create Message Box
    mbox.show(exc);                     //
    return false;                       //Return error
  }                                     //
  updateStatus();                       //Update status
  if (hot)                              //Is hot link active?
  {
    log("<<hotLinkUpdate: HotWork sent to client\n");
    server.hotLinkUpdate("HotWork");    //Update work hot link
    log("<<hotLinkUpdate: HotDone sent to client\n");
    server.hotLinkUpdate("HotDone");    //Update done hot link
  } /* endif */
  return true;
} /* end AServerWindow :: addDone(..) */

//**************************************************************************
// AServerWindow :: addTodo(..)
//**************************************************************************
Boolean AServerWindow::addTodo(char * newTodo)
{
  todoList.addAsLast(newTodo);          //Add new todo
  return true;
} /* end AServerWindow :: addTodo(..) */

//**************************************************************************
// AServerWindow :: moreTodo()
//**************************************************************************
Boolean AServerWindow::moreTodo()       //Add more items to the todo list
{
  addTodo("Buy Birthday Gifts");        //Add item to todo list
  addTodo("Pay Phone Bill");            //
  addTodo("Pay Taxes");                 //
  addTodo("Take out the papers");       //
  addTodo("Take out the trash");        //
  addTodo("Wash the car");              //
  addTodo("Wash the dishes");           //
  todoList.select(0);                   //Set the first item selected
  return true;                          //
} /* end AServerWindow :: moreTodo() */

//**************************************************************************
// AServerWindow :: addHot()
//**************************************************************************
Boolean AServerWindow::addHot()
{
  hot=hot+1;                            //Set new value
  if (hot == 1)
  {
    infoArea.setInactiveText(STR_HOTYES);//Set information area text from RC
  } /* endif */
  return true;                          //
} /* end AServerWindow :: addHot() */

//**************************************************************************
// AServerWindow :: removeHot()
//**************************************************************************
Boolean AServerWindow::removeHot()
{
  hot=hot-1;                            //Set new value
  if (hot == 0)
  {
    infoArea.setInactiveText(STR_HOTNO);//Set information area text from RC
  } /* endif */
  return true;                          //
} /* end AServerWindow :: removeHot(..) */

//**************************************************************************
// AServerWindow :: nextTodo()
//**************************************************************************
IString AServerWindow::nextTodo()
{
  IListBox::Cursor lbCursor(todoList);  //List Box Cursor
  lbCursor.setToFirst();                //Set to first item selected
  IString temp(                         //Set temp to text of 1st item selected
    todoList.elementAt(lbCursor));      //
  workList.addAsLast(temp);             //Add to work list
  todoList.removeAt(lbCursor);          //Remove this item
  if (todoList.count() == 0)            //If zero count then add more items
    moreTodo();                         //  since the work is never done
  todoList.select(0);                   //Set the selection to first
  if (hot)                              //Is hot link active?
  {
    log("<<hotLinkUpdate: HotTodo sent to client\n");
    server.hotLinkUpdate("HotTodo");    //Update todo hot link
    log("<<hotLinkUpdate: HotWork sent to client\n");
    server.hotLinkUpdate("HotWork");    //Update work hot link
  } /* endif */
  updateStatus();                       //Update the status line
  return temp;                          //Return String
} /* end AServerWindow :: nextTodo() */

//**************************************************************************
// AServerWindow :: todo()
//**************************************************************************
IString AServerWindow::todo()
{
  IString temp(todoList.count());       //Get count
  return temp;                          //Return String
} /* end AServerWindow :: todo() */

//**************************************************************************
// AServerWindow :: work()
//**************************************************************************
IString AServerWindow::work()
{
  IString temp(workList.count());       //Get count
  return temp;                          //Return String
} /* end AServerWindow :: work() */

//**************************************************************************
// AServerWindow :: done()
//**************************************************************************
IString AServerWindow::done()
{
  IString temp(doneList.count());       //Get count
  return temp;                          //Return String
} /* end AServerWindow :: done() */

//**************************************************************************
// AServerWindow :: updateStatus()
//**************************************************************************
Boolean AServerWindow::updateStatus()
{
  todoStatus.setText(                   //Set the status to todo count
    IString("Todo count: ")+
    IString(todoList.count()));
  workStatus.setText(                   //Set the status to work count
    IString("Work count: ")+
    IString(workList.count()));
  doneStatus.setText(                   //Set the status to done count
    IString("Done count: ")+
    IString(doneList.count()));

  return true;
} /* end AServerWindow :: updateStatus() */


//**************************************************************************
// ATopicServer :: ATopicServer - Constructor for our DDE server class
//**************************************************************************
ATopicServer :: ATopicServer(           //Constructor for ATopicServer
                 const char* app,       //Application Name
                 const char* topic,     //Topic Name
                 AServerWindow* mainWin)//Windower
              : IDDETopicServer(        //Call IDDETopicServer constructor
                 app,                   //
                 topic,                 //
                 mainWin, false),       //
           serverWindow(mainWin)        //Set window
{;} /* end ATopicServer :: ATopicServer(..) */

//**************************************************************************
// ATopicServer :: requestHotLinkData(..)
//**************************************************************************
void ATopicServer :: requestHotLinkData(IDDERequestDataEvent& event)
{
  Boolean rc=true;
  if(event.item()==IString("HotTodo"))  //Check the requested item
  {                                     //  the request is ok
    rc=false;
    IString todo;                       //  Define string to return
    todo=mainWindow()->todo();          //  Get todo count from window
    mainWindow()->log(                  //  Log this request with
      IString( ">>requestHotLineData: ")+//   function member name
      event.item()+                     //    event item (todo)
      IString(" Returning: ")+todo+     //    returned to client
      IString("\n"));                   //
    event.setData((char*)todo);         //  Set data to return to client
  }
  if(event.item()==IString("HotWork"))  //Check the requested item
  {                                     //  the request is ok
    rc=false;
    IString work;                       //  Define string to return
    work=mainWindow()->work();          //  Get work count from window
    mainWindow()->log(                  //  Log this request with
      IString(">>requestHotLineData: ")+//   function member name
      event.item()+                     //    event item (work)
      IString(" Returning: ")+work+     //    returned to client
      IString("\n"));                   //
    event.setData((char*)work);         //  Set data to return to client
  }
  if(event.item()==IString("HotDone"))  //Check the requested item
  {                                     //  the request is ok
    rc=false;
    IString done;                       //  Define string to return
    done=mainWindow()->done();          //  Get done count from window
    mainWindow()->log(                  //  Log this request with
      IString( ">>requestHotLineData: ")+//   function member name
      event.item()+                     //    event item (done)
      IString(" Returning: ")+done+     //    returned to client
      IString("\n"));                   //
    event.setData((char*)done);         //  Set data to return to client
  }
  if (rc)                               //Request does not match
  {                                     //
    mainWindow()->log(
      IString( "**>>requestHotLineData: ")+//   function member name
      IString("Unable to provide item: ") +
      event.item() + "\n");
  }
} /* end ATopicServer :: requestHotLinkData(..) */

//**************************************************************************
// ATopicServer :: beginHotLink(..)
//**************************************************************************
Boolean ATopicServer :: beginHotLink(unsigned long conversationId,
                          IDDEServerHotLinkEvent& event)
{
  Boolean rc=false;                     //Define return code
  if(event.item()==IString("HotTodo"))  //Check the requested item
  {                                     //  the request is ok
    rc=true;                            //
    mainWindow()->log(                  //  Log this request with
      IString( ">>beginHotLink: ")+     //    function member name
      event.item()+                     //    event item
      IString(" From: ")+               //
      IString(conversationId)+          //    conversation ID
      IString("\n"));                   //
  }
  if(event.item()==IString("HotWork"))  //Check the requested item
  {                                     //  the request is ok
    mainWindow()->log(                  //  Log this request with
      IString( ">>beginHotLink: ")+     //    function member name
      event.item()+                     //    event item
      IString(" From: ")+               //
      IString(conversationId)+          //    conversation ID
      IString("\n"));                   //
    rc=true;                            //
  }
  if(event.item()==IString("HotDone"))  //Check the requested item
  {                                     //  the request is ok
    mainWindow()->log(                  //  Log this request with
      IString( ">>beginHotLink: ")+     //    function member name
      event.item()+                     //    event item
      IString(" From: ")+               //
      IString(conversationId)+          //    conversation ID
      IString("\n"));                   //
    rc=true;                            //
  }
  if (!rc)                              //Request does not match
  {                                     //
    mainWindow()->log(
      IString( "**>>beginHotLink: ")+   //    function member name
      IString("Unable to handle item: ")+//
      event.item() + "\n");             //
  }
  else
  {
    mainWindow()->addHot();             //Set hot link active
  }
  return rc;
} /* end ATopicServer :: beginHotLink(..) */

//**************************************************************************
// ATopicServer :: hotLinkEnded(..)
//**************************************************************************
void ATopicServer :: hotLinkEnded(unsigned long conversationId,
                          IDDEEvent& event)
{
  mainWindow()->log(                  //  Log this request with
    IString( ">>hotLinkEnded: ")+     //    function member name
    event.item()+                     //    event item
    IString(" From: ")+               //
    IString(conversationId)+          //    conversation ID
    IString("\n"));                   //
  mainWindow()->removeHot();          //Set hot link inactive
  mainWindow()->removeHot();          //Set hot link inactive
  mainWindow()->removeHot();          //Set hot link inactive
} /* end ATopicServer :: beginHotLink(..) */

//**************************************************************************
// ATopicServer :: pokeData(..)
//**************************************************************************
Boolean ATopicServer :: pokeData(unsigned long conversationId,
                          IDDEPokeEvent& event)
{
  int rc=1;                             //Set default return code to processed
  if(event.item()==IString("DonePoke")) //Check the requested item
  {                                     //  the request is ok
    IString workItem=event.pokedData(); //Get poked Data
    mainWindow()->log(                  //  Log this request with
      IString(">>pokeData: ")+          //    function member name
      event.item()+                     //    event item (nextTodo)
      IString(" Data: ")+workItem+      //    work item from client
      IString(" From: ")+               //
      IString(conversationId)+          //    conversation ID
      IString("\n"));                   //
    mainWindow()->addDone(workItem);    //Add this item to the done list
  }
  else                                  //Request does not match
  {                                     //
    mainWindow()->log(
      IString("**>>pokeData: ")+
      IString("Unable to handle item: ") +
      event.item() + "\n");
    rc=0;                               //Set return code to event not processed
  }
  return rc;
} /* end ATopicServer :: pokeData(..) */

//**************************************************************************
// ATopicServer :: requestData(..)
//**************************************************************************
Boolean ATopicServer :: requestData(unsigned long conversationId,
                               IDDERequestDataEvent& event)
{
  int rc=false;                         //Set default return code to processed
  if(event.item()==IString("NextTodo")) //Check the requested item
  {                                     //  the request is ok
    rc=true;
    IString workItem;                   //  Define work item
    workItem=mainWindow()->nextTodo();  //  Get work item from todo list
    mainWindow()->log(                  //  Log this request with
      IString( ">>requestData: ")+      //    member name
      event.item()+                     //    event item (nextTodo)
      IString(" Returning: ")+workItem+ //    work item returned to client
      IString(" To: ")+                 //
      IString(conversationId)+          //    conversation ID
      IString("\n"));                   //
    event.setData((char*)workItem);     //  Set data to return to client
  }
  if(event.item()==IString("HotTodo"))  //Check the requested item
  {                                     //  the request is ok
    rc=true;
    IString todo;                       //  Define string to return
    todo=mainWindow()->todo();          //  Get todo count from window
    mainWindow()->log(                  //  Log this request with
      IString( ">>requestData: ")+      //   function member name
      event.item()+                     //    event item (todo)
      IString(" Returning: ")+todo+     //    returned to client
      IString("\n"));                   //
    event.setData((char*)todo);         //  Set data to return to client
  }
  if(event.item()==IString("HotWork"))  //Check the requested item
  {                                     //  the request is ok
    rc=true;
    IString work;                       //  Define string to return
    work=mainWindow()->work();          //  Get work count from window
    mainWindow()->log(                  //  Log this request with
      IString(">>requestData: ")+       //   function member name
      event.item()+                     //    event item (work)
      IString(" Returning: ")+work+     //    returned to client
      IString("\n"));                   //
    event.setData((char*)work);         //  Set data to return to client
  }
  if(event.item()==IString("HotDone"))  //Check the requested item
  {                                     //  the request is ok
    rc=true;
    IString done;                       //  Define string to return
    done=mainWindow()->done();          //  Get done count from window
    mainWindow()->log(                  //  Log this request with
      IString( ">>requestData: ")+      //   function member name
      event.item()+                     //    event item (done)
      IString(" Returning: ")+done+     //    returned to client
      IString("\n"));                   //
    event.setData((char*)done);         //  Set data to return to client
  }
  if (!rc)                              //Request does not match
  {                                     //
    mainWindow()->log(
      IString("**>>requestData: ")+
      IString("Unable to provide item: ")+
      event.item() + "\n");
  }
  return rc;
} /* end ATopicServer :: requestData(..) */

//**************************************************************************
// ATopicServer :: acceptConversation(..)
//**************************************************************************
Boolean ATopicServer :: acceptConversation(unsigned long conversationId,
                                      IDDEBeginEvent& event)
{
  mainWindow()->log(                    //  Log this message with
    IString( ">>acceptConversation with: ")+//member function name
    IString(conversationId)+            //    conversation ID
    IString("\n"));                     //
  return true;
} /* end ATopicServer :: acceptConversation(..) */

//**************************************************************************
// ATopicServer :: conversationEnded(..)
//**************************************************************************
void ATopicServer :: conversationEnded(unsigned long conversationId,
                                   IDDEEndEvent& endEvt)
{
  if (endEvt.sourceOfEnd() == IDDEEndEvent::server)
    mainWindow()->log(                  //Log this message with
      IString(">>conversationEnded with: ")+//member function name
      IString(conversationId)+          //     conversation ID
      IString(" by the Server\n"));     //     by the Server

  else if (endEvt.sourceOfEnd() == IDDEEndEvent::client)
    mainWindow()->log(                  //Log this message
      IString(">>conversationEnded with: ")+//member function name
      IString(conversationId)+          //     conversation ID
      IString(" by the Client\n"));     //     by the Client

  else
    mainWindow()->log(                  //Log this message
      IString("**>>conversationEnded with: ")+//member function name
      IString(conversationId)+          //     conversation ID
      IString(" due to an error\n"));   //     due to an error

} /* end ATopicServer :: conversationEnded(..) */
