extern "C"
  {
  #include <stdio.h>
  #include <sys\stat.h>
  #define INCL_GPIBITMAPS
  #include <os2.h>
  }

#include <iostream.h>
#include <fstream.h>

#include <iframe.hpp>
#include <ibmpctl.hpp>
#include <istring.hpp>
#include <idmhndlr.hpp>

#include "dmsamp2.hpp"

/*------------------------------------------------------------------------------
| main                                                                         |
------------------------------------------------------------------------------*/
void main()
{
  /***********************************************************************/
  /* Create a generic frame window.                                      */
  /***********************************************************************/
  IFrameWindow
    frame ( "C Set ++ Direct Manipulation - Sample 2" );

  /***********************************************************************/
  /* Create an empty bitmap control.                                     */
  /***********************************************************************/
  IBitmapControl
    bmpControl ( 0, &frame, &frame );

  /***********************************************************************/
  /* Create a target handler for the bitmap control and use default      */
  /* C Set++ UI renderers.                                               */
  /***********************************************************************/
  IDMHandler::enableDropOn( &bmpControl );

  /***********************************************************************/
  /* Create bitmap drag item provider.                                   */
  /***********************************************************************/
  ABitmapProvider
    itemProvider;

  /***********************************************************************/
  /* Attach provider to the bitmap control.                              */
  /***********************************************************************/
  bmpControl.setItemProvider( &itemProvider );

  /***********************************************************************/
  /* Set the bitmap's control as the frame's client                      */
  /* and display the frame.                                              */
  /***********************************************************************/
  bmpControl.setText( "Drop .bmp files here." );
  frame.setClient( &bmpControl )
       .showModally();
}

/*------------------------------------------------------------------------------
| ABitmapItem::ABitmapItem                                                     |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
ABitmapItem :: ABitmapItem ( const IDMItem::Handle &item )
  : IDMItem( item )
{
}

/*------------------------------------------------------------------------------
| ABitmapItem::targetDrop                                                      |
|                                                                              |
| Take the dropped file, create a PM bitmap object,                            |
| and set it into the target window.                                           |
------------------------------------------------------------------------------*/
Boolean ABitmapItem :: targetDrop ( IDMTargetDropEvent & )
{
  /***********************************************************************/
  /* Get pointer to the target bitmap control.                           */
  /***********************************************************************/
  IBitmapControl
   *bmpControl = (IBitmapControl*)this->targetOperation()->targetWindow();

  /***********************************************************************/
  /* Construct dropped .BMP file name from this drag item.               */
  /***********************************************************************/
  IString
    fname = this->containerName() + "\\" + this->sourceName();

  /***********************************************************************/
  /* Get file size.                                                      */
  /***********************************************************************/
  struct stat
    buf;
  stat( fname, &buf);

  /***********************************************************************/
  /* Open and read the file.                                             */
  /***********************************************************************/
  FILE
   *fileptr = fopen( fname, "rb" );
  char
   *buffer = new char[ buf.st_size ];
  fread( buffer, sizeof(char), buf.st_size, fileptr );

  /***********************************************************************/
  /* Construct the bitmap from the file.                                 */
  /***********************************************************************/
  BITMAPARRAYFILEHEADER2
   *array = ( BITMAPARRAYFILEHEADER2 * )buffer;
  BITMAPFILEHEADER2
   *header;

  /***********************************************************************/
  /* First, see if the file holds array of bitmaps.                      */
  /***********************************************************************/
  if ( array->usType == BFT_BITMAPARRAY )
  {
    /*********************************************************************/
    /* It is, point to first file header in the array.                   */
    /*********************************************************************/
    header = &array->bfh2;
  }
  else
  {
    /*********************************************************************/
    /* It isn't, point to file header at the start of the file.          */
    /*********************************************************************/
    header = ( BITMAPFILEHEADER2 * )buffer;
  }

  /***********************************************************************/
  /* Now check to see if this is a bitmap.                               */
  /***********************************************************************/
  if ( header->usType == BFT_BMAP)
  {
    /*********************************************************************/
    /* We can proceed, first get a presentation space.                   */
    /*********************************************************************/
    IPresSpaceHandle
      hps = bmpControl -> presSpace();

    if ( hps )
    {
      /*******************************************************************/
      /* Now create the bitmap from the file contents.                   */
      /*******************************************************************/
      IBitmapHandle
        hbm = GpiCreateBitmap( hps,
                               &header->bmp2,
                               CBM_INIT,
                               (PBYTE)buffer + header->offBits,
                               (BITMAPINFO2*)&header->bmp2 );
      if ( hbm )
      {
        /*****************************************************************/
        /* Get previously dropped bitmap.                                */
        /*****************************************************************/
        IBitmapHandle
          old = bmpControl -> bitmap();

        /*****************************************************************/
        /* Set new one.                                                  */
        /*****************************************************************/
        bmpControl -> setBitmap( hbm );

        /*****************************************************************/
        /* Destroy old one since we no longer need it.                   */
        /*****************************************************************/
        GpiDeleteBitmap( old );

        /*****************************************************************/
        /* Indicate name of dropped file.                                */
        /*****************************************************************/
        bmpControl -> setText( fname );
      }
      else
      {
        bmpControl -> setText( "Couldn't create bitmap!" );
      }
      // Release the presentation space...
      bmpControl -> releasePresSpace(hps);
    }
    else
    {
      bmpControl -> setText ( "Couldn't get PS!" );
    }
  }
  else
  {
    bmpControl -> setText( fname + " isn't a bitmap!" );
  }

  /***********************************************************************/
  /* Free the buffer.                                                    */
  /***********************************************************************/
  delete [] buffer;

  return true;
}

/*------------------------------------------------------------------------------
| ABitmapProvider::providerEnterSupport                                        |
|                                                                              |
| Verify that we're dealing with a bitmap object before                        |
| allowing a drop.                                                             |
------------------------------------------------------------------------------*/
Boolean ABitmapProvider :: provideEnterSupport ( IDMTargetEnterEvent &event )
{
  /***********************************************************************/
  /* Get handle to the drag target operation                             */
  /***********************************************************************/
  IDMTargetOperation::Handle targetOp = IDMTargetOperation::targetOperation();

  /***********************************************************************/
  /* Get the types for the drag item.                                    */
  /***********************************************************************/
  IString strTypes = targetOp->item(1)->types();

  /***********************************************************************/
  /* If type is either "Bitmap" or "Plain Text" (used by WPS), we can    */
  /* display the drag item.                                              */
  /***********************************************************************/
  if ((strTypes.indexOf(IDM::bitmap))  ||
      (strTypes.indexOf(IDM::plainText)))
  {
    return(true);
  }

  /***********************************************************************/
  /* Type is not recognized - set the drop indicator to prevent a drop!  */
  /***********************************************************************/
  event.setDropIndicator(IDM::notOk);
  return(true);
}

