#ifndef ACOLORWINDOW_HPP
#define ACOLORWINDOW_HPP
/******************************************************************************/
/* HELLO WORLD SAMPLE PROGRAM - Version 6: AColorWindow Class  (ACOLORW6.HPP) */
/*                                                                            */
/* COPYRIGHT: Copyright (C) International Business Machines Corp., 1992,1993. */
/*                                                                            */
/* DISCLAIMER OF WARRANTIES:                                                  */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is not part of any standard IBM product   */
/*   and is provided to you solely for the purpose of assisting you in the    */
/*   development of your applications.  The code is provided "AS IS",         */
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */
/******************************************************************************/

//**************************************************************************
// The entire file was created at version 6                                *
//**************************************************************************

#include <iframe.hpp>                   //IFrameWindow Class (Parent)
#include <icmdhdr.hpp>                  //ICommandHandler (Parent)

class IEntryField;
class INotebook::PageSettings;
class INotebook;
class IMultiCellCanvas;
class IRadioButton;
#include <iradiobt.hpp>                 //IRadioButton
class IPushButton;
class IListBox;

//**************************************************************************
// Class:   AColorWindow                                                   *
//                                                                         *
// Purpose: Dialog window for the C++ Hello World sample application.      *
//          It is a subclass of IFrameWindow, ICommandHandler              *
//                                                                         *
//**************************************************************************
class AColorWindow : public IFrameWindow, public ICommandHandler
{
public:
  AColorWindow (                        //Construtor for "Page" Window
    IStaticText * staticWindow,
    IListBox * listWindow,
    IWindow * ownerWnd);
  ~AColorWindow();                      //Destructor

protected:

class AColorButton : public IRadioButton//Create AColorButton Class
{
public:
  AColorButton(unsigned long windowId,
               IWindow * parentOwner,
               unsigned long stringId);
} ;

  virtual Boolean
    command(ICommandEvent& cmdevt);     //Process the dialog command events
  virtual Boolean createStaticPage();   //Create the Static Color Page
  virtual Boolean createListPage();     //Create the ListBox Color Page

  virtual Boolean applyStatic();        //Apply Changes from Static Page
  virtual Boolean applyList();          //Apply Changes from List Page

private:
  IStaticText      * staticTarget;      //Target Window for Color Changes
  IListBox         * listTarget;        //Target Window for Color Changes
  INotebook        * notebook;          //Notebook Window
  INotebook::PageSettings staticPage;   //Static Color Page
  INotebook::PageSettings listPage;     //ListBox Color Page
  IMultiCellCanvas * staticCanvas;      //Static Color Canvas
  IMultiCellCanvas * listCanvas;        //ListBox Color Canvas

  AColorButton     * staticWhite;       //Static White Radio Button
  AColorButton     * staticBlack;       //Static Black Radio Button
  AColorButton     * staticBlue;        //Static Blue Radio Button
  AColorButton     * staticRed;         //Static Red Radio Button
  AColorButton     * staticPink;        //Static Pink Radio Button
  AColorButton     * staticGreen;       //Static Green Radio Button
  AColorButton     * staticCyan;        //Static Cyan Radio Button
  AColorButton     * staticYellow;      //Static Yellow Radio Button

  IPushButton      * staticApply;       //Static Apply Button
  IPushButton      * staticCancel;      //Static Cancel Button

  AColorButton     * listWhite;         //ListBox White Radio Button
  AColorButton     * listBlack;         //ListBox Black Radio Button
  AColorButton     * listBlue;          //ListBox Blue Radio Button
  AColorButton     * listRed;           //ListBox Red Radio Button
  AColorButton     * listPink;          //ListBox Pink Radio Button
  AColorButton     * listGreen;         //ListBox Green Radio Button
  AColorButton     * listCyan;          //ListBox Cyan Radio Button
  AColorButton     * listYellow;        //ListBox Yellow Radio Button

  IPushButton      * listApply;         //ListBox Apply Button
  IPushButton      * listCancel;        //ListBox Cancel Button

}; // AColorWindow
#endif
